package units
{
	import com.greensock.TweenMax;
	import com.greensock.motionPaths.LinePath2D;
	
	import flash.desktop.Icon;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.geom.Point;
	import flash.utils.ByteArray;
	
	import connections.PlayerIOConnection;
	
	import feathers.controls.Button;
	
	import maps.GameDataType;
	import maps.Map;
	import maps.TerrainState;
	import maps.TerrainType;
	
	import starling.animation.IAnimatable;
	import starling.core.Starling;
	import starling.display.Image;
	import starling.display.MovieClip;
	import starling.display.Sprite;
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.filters.BlurFilter;
	import starling.textures.Texture;
	import starling.utils.Line;
	
	import treefortress.sound.SoundAS;
	
	public class Bomber extends Unit
	{
		protected var isPumpingWater:Boolean;
		protected var bucket:Image;
		protected var bucketWidth:Number = 50;
		protected var speedWaterPump:Number = 1/30/10;		// 1 iteration / 30 fps / X sec = incremental delta of pumping water for X secs
		protected var speedWaterDrop:Number = 1/30/4.5;		// 1 iteration / 30 fps / X sec = decremental delta of dropping water over X secs;
		protected var waterDropCounter:int = 0; 			// incremental counter used with "waterDropFrequency"
		protected var waterDropFrequency:int = 10;			// to drop water 1 per second, make it 30 since there's 30fps
		protected var pumpWaterPointArray:Array;
		protected var dropWaterPointArray:Array;
		protected var shadowDistance:Number = 1;
		
		private var clearPumpZone:Button;
		
		private var pumpZone:Sprite = new Sprite();
		private var isFireUnderUnit:Boolean = false;
		
		
		
		
		public function Bomber( _map:Map, _isMyUnit:Boolean, _id:int)
		{
			type = UnitType.BOMBER;
//			TweenPlugin.activate([DropShadowFilterPlugin]);
			
			super(_map, _isMyUnit, _id);
		}
		
		override public function toggleNight():void
		{
			var currentScale:Number = 1;
			if (bucket) 
			{
				currentScale = bucket.scaleX;
				bucket.removeFromParent(true);
			}
			bucket = new Image(Assets.getAtlas().getTexture((map.isNight) ? "BucketBomberNight":"BucketBomber"));
			bucket.alignPivot();
			bucket.scaleX = bucket.scaleY = currentScale;
			
			super.initializeUnitIcon((map.isNight) ? "iconBomberNight" : "iconBomberDay", true, 24);
			
			iconUnitContainer.addChild(unitIconShadow);			
			iconUnitContainer.addChild(bucket);
			addChild(unitIconCollision);
			iconUnitContainer.addChild(unitIcon);

		}	
			
		override protected function initializeOnce():void
		{
			super.initializeOnce();
			
			bucket = new Image(Assets.getAtlas().getTexture((map.isNight) ? "BucketBomberNight":"BucketBomber"));
			bucket.alignPivot();
			
			speed = 1.5;
			
			super.initializeUnitIcon((map.isNight) ? "iconBomberNight" : "iconBomberDay", true, 24);
			
			addNewPointType("BomberPumpZone", true, 5);
			pointType["BomberPumpZone"].x = 0;
			addNewPointType("BomberDropZone", true, 5);
			pointType["BomberDropZone"].x = -25;
			
			//pumpZone.addChild(pointType["BomberPumpZone"]);
			//pumpZone = new Sprite();
			map.lineLayer.addChild(pumpZone);			
			//var dropShadow:BlurFilter = BlurFilter.createDropShadow(15);
			
			//this.filter = dropShadow;
			
//			TweenMax.to(this,0,{dropShadowFilter:{blurX:5, blurY:5, distance:5, alpha:0.6}});
			clearPumpZone = new Button();
			clearPumpZone.label = "Clear PumpZone";
			clearPumpZone.addEventListener(starling.events.Event.TRIGGERED, onButtonClicked);
			containerMenu.addChild(clearPumpZone);
			
			unitIconShadow.x = 20 * shadowDistance;
			unitIconShadow.y = 20 * shadowDistance;
			
		}
		
		override public function initializeUnit():void
		{
			super.initializeUnit();
			
			callout("Place me anywhere", this, true, false);
			iconUnitContainer.addChild(unitIconShadow);			
			iconUnitContainer.addChild(bucket);
			addChild(unitIconCollision);
			iconUnitContainer.addChild(unitIcon);
			
			bucket.scaleX = bucket.scaleY = 0;
//			pumpWaterPointArray = new Array();
			dropWaterPointArray = new Array();
			for (var i:int = 0; i < 4; i++)
			{
				dropWaterPointArray.push(new MovieClip(Assets.getAtlas().getTextures("BomberDropCircle_"),1));
				TweenMax.delayedCall(1-0.1*i,starling.core.Starling.juggler.add,[dropWaterPointArray[i]]);
				
				map.lineLayer.addChild(dropWaterPointArray[i]);
				dropWaterPointArray[i].visible = false;
			}
		}
		
		private function onButtonClicked(e:starling.events.Event):void
		{
			var buttonClicked:Button = e.target as Button;
			
			if (buttonClicked == clearPumpZone)
			{
				pumpZone.removeChildren();
				clearPumpZone.isEnabled = false;
				sendUpdatedUnit(true);
			}
		}
		/*
		override protected function drawLine():void
		{
			if (linePath.points.length > 0)
			{
				//				var tempLinePath:LinePath2D
				var points:Number = linePath.totalLength / gap;
				var num:Number = 1 / points;
				var angle:Number;
				
				//spread the points evenly  
				var p:Number = 0;
				var point:Point;
				var prevPoint:Point;
				
				//var currentDot:int = 0;
				
				var upToPoints:Array = linePath.getAllPointsUpToProgress(progress);
				var afterPoints:Array = linePath.getAllPointsAfterProgress(progress);
				
				for (var i:int = 0; i < upToPoints.length; i++)
				{
					if (map.lineLayer.contains(linePointArray[i])) map.lineLayer.removeChild(linePointArray[i]);
				}
				
				for (var i:int = 0; i < afterPoints.length - 1; i++)
				{
//					if (!map.lineLayer.contains(linePointArray[i])) 
//					{
						linePointArray[i].x = afterPoints[i].x - 8;
						linePointArray[i].y = afterPoints[i].y - 8;
						map.lineLayer.addChild(linePointArray[i]);
//					}
				}
				
				
//				var totalSum:Number = (isCollect) ? 1 : 1-num;
//				while (p < totalSum)
//				{
//					
//					//check p against progress so we don't draw the dashes/dots the plane has moved past already  
//					if (p > progress && linePointArray[currentDot])
//					{
//						point = linePath.getPointAtProgress(p);
//						linePointArray[currentDot].x = point.x - 8;
//						linePointArray[currentDot].y = point.y - 8;
//						if (!map.lineLayer.contains(linePointArray[currentDot])) map.lineLayer.addChild(linePointArray[currentDot]);
//						currentDot++;
//						prevPoint = point;
//					}
//					p += num;
//				}
//
//						
//				
//				for (var i:int = currentDot; i < numDots; i++)
//				{
//					if (map.lineLayer.contains(linePointArray[i])) map.lineLayer.removeChild(linePointArray[i]);
//				}
				
			}
		}
		*/
		
		override protected function updatePointTypeByTerrain():void
		{
			try
			{
				var terrainType:Number = map.terrain[Math.round(targetPoint.x/map.blitSize)][Math.round(targetPoint.y/map.blitSize)].type;
				if (terrainType == TerrainType.WATER_DEEP && bucket.scaleX < 1)
				{
					showPointType("BomberPumpZone");

				}
//				else if (map.checkToSeeIfThereIsFire(targetPoint, bucketWidth*bucket.scaleX))
//				{
//					showPointType("BomberDropZone");
//				}
				else
				{
					showPointType("targetNormal");
				}
			}
			catch(e:Error)
			{
				showPointType("targetNormal");
			}
		}
		

		
		
		/**
		 * 
		 * @param	unit
		 */
		override protected function arrivedAtDestination ():void
		{
			super.arrivedAtDestination();
			
			clearLine();
			
//			if ()
//			{
//				map.lineLayer.addChild(pumpZone);
//			}
			trace("Bomber.arrivedAtDestination();");
			trace("> state: " + state);
//			if (bucket.scaleX > 0 && map.checkToSeeIfThereIsFire(this, bucketWidth))
//			{
////				trace("> there is fire here");
////				state = UnitState.WATER_DROP;
////				if (redblox.isActive) SoundAS.play("WaterBomberDropWater").fadeTo(0,7000);
//
//			}
//			else 
			if (bucket.scaleX < 1 && map.terrainTypeUnderPoint(this, TerrainType.WATER_DEEP))
			{
				trace("> there is water here");
				
				if (redblox.isActive) SoundAS.play("WaterBomberPumpZone");//.fadeTo(0,10000);
				if (pumpZone.numChildren == 0)
				{
					pumpZone.x = targetPoint.x;
					pumpZone.y = targetPoint.y;
					pumpZone.rotation = targetPoint.rotation;
					//if (isMyUnit) 
						pumpZone.addChild(pointType["BomberPumpZone"]);
					
					TweenMax.to(pumpZone, 0.25, {scaleY:0});
					TweenMax.to(pumpZone, 0.25, {delay:0.25, scaleY:1, overwrite:false});
					callout("The PUMP ZONE \nis now set \nfor auto-return.", pumpZone, false, false);
				}
				else if (pumpZone.numChildren != 0)
				{
					targetPoint.x = pumpZone.x;
					targetPoint.y = pumpZone.y;
					targetPoint.rotation = pumpZone.rotation;
					
				}
				if (isMyUnit) pumpZone.alpha = 0.7;
				else pumpZone.alpha = 0.3;
				state = UnitState.WATER_PUMP;
			}
			else if (bucket.scaleX == 0)
			{
				if (pumpZone.numChildren != 0)
				{
					progress = 0;
					linePath.points = new Array();
					newLinePathPoints = new Array();
					updateSpeedAndProgress();
					linePath.appendPoint(new Point(x, y));
					calculatePoints(new Point(x, y), new Point(pumpZone.x, pumpZone.y), linePath);
					linePath.appendPoint(new Point(pumpZone.x, pumpZone.y));
					updateSpeedAndProgress();
					pumpZone.alpha = 1;
					targetPoint.x = pumpZone.x;
					targetPoint.y = pumpZone.y;
					targetPoint.rotation = pumpZone.rotation;
					sendUpdatedUnit(true);
				}
			}
			
		} 
		
		override public function advanceTime(passedTime:Number):void
		{
			//trace("Bomber.advanceTime(): " + isCollect);
			super.advanceTime(passedTime);
			super.rotateUnitAlongPath();
			
			if (isDragAndDrop || state == UnitState.DEAD)
			{
				return;
			}
			
			if (terrainUnderUnit == null)
			{
				state = UnitState.OFFSCREEN;
//				if (terrainUnderUnitPrevious != null) 
//					callout("Off-Screen PUMP ZONE takes 50 seconds to fill up.", this, true);
				
				
			}
			else if (terrainUnderUnit && state == UnitState.OFFSCREEN)
			{
				state = UnitState.NORMAL;
				
			}
			
			if (progress + speedOnPath >= 1 || linePath.points.length <= 1 || linePath.totalLength < 5) {  
//				lines.graphics.clear();
				//if (!isOffScreen && panel.panelState.visible == false)
//				if (!isOffScreen && !isWaitingToLaunch)
				if (this.x > -50 && this.x < map.terrainWidth+50 && y > -50 && y < map.terrainHeight+50)
				{
					//trace("unitIcon.rotation: " + unitIcon.rotation);
					//trace("targetPoint.rotation: " + targetPoint.rotation);
					if (!isNaN(targetPoint.rotation)) 
					{
						targetRotation = targetPoint.rotation * 180/Math.PI;
					}
					
					rotateUnitAlongPath();
					
					var angle:Number = unitIcon.rotation; // * Math.PI / 180;
					
					//var lastPoint:Point = points[points.length-1]; 
					//if (redblox.fps < 20)
					//{
					//this.x += speed * Math.cos(angle) * timeScale * TweenMax.globalTimeScale; 
					//this.y += speed * Math.sin(angle) * timeScale * TweenMax.globalTimeScale;
					//}
					//else
					//{
					this.x += speed * (passedTime * 30) * Math.cos(angle) * TweenMax.globalTimeScale;
					this.y += speed * (passedTime * 30) * Math.sin(angle) * TweenMax.globalTimeScale;
					//}
					//endPoint.visible = false;
				}
			} 
			
			
			if (state == UnitState.WATER_PUMP)
			{
				waterPump(passedTime);
				if (shadowDistance > 0.05){
					shadowDistance = Math.max(0.05, shadowDistance - 0.01);
					unitIconShadow.x = unitIconShadow.y = 20 * shadowDistance;
				}
			}
			else if (state == UnitState.OFFSCREEN)
			{
				if (!this.contains(countdownPopup)) addChild(countdownPopup);
				
				
				if (bucket.scaleX < 1)
				{
					waterPump(passedTime);
				}
				if (bucket.scaleX == 1)
				{
					shadowDistance = Math.max(0.05, shadowDistance + 0.01);
					unitIconShadow.x = unitIconShadow.y = 20 * shadowDistance;
				}
				if (shadowDistance > 0.05){
					shadowDistance = Math.max(0.05, shadowDistance - 0.01);
					unitIconShadow.x = unitIconShadow.y = 20 * shadowDistance;
				}
			}
			else if (state == UnitState.WATER_DROP)
			{
				waterDrop(passedTime);
			}
			else if (state == UnitState.NORMAL)
			{
				if (this.contains(countdownPopup)) removeChild(countdownPopup);
				if (shadowDistance < 1){
					shadowDistance = Math.min(1, shadowDistance + 0.01);
					unitIconShadow.x = unitIconShadow.y = 20 * shadowDistance;
				}
			}
		}
		
//		protected function pickupUnit(targetUnit:Sprite):void
//		{
//			// TODO Auto Generated method stub
//			
//		}
//		
		protected function waterPump(passedTime:Number):void
		{
			//trace("waterPump()");
			
			if (map.terrainTypeUnderPoint(this, TerrainType.WATER_DEEP) && bucket.scaleX < 1)
			{
				bucket.scaleX = bucket.scaleY = bucket.scaleX + speedWaterPump * passedTime * 30 * TweenMax.globalTimeScale;
//				udpateCountdown((10 - Math.round(bucket.scaleX * 10)) + "s to FULL");
			}
			else if (state == UnitState.OFFSCREEN && bucket.scaleX < 1)
			{
				bucket.scaleX = bucket.scaleY = bucket.scaleX + (speedWaterPump/5) * passedTime * 30 * TweenMax.globalTimeScale;
				udpateCountdown((50 - Math.round(bucket.scaleX * 50)) + "s to FULL");
			}
			else if (state == UnitState.OFFSCREEN)
			{
			}
			else
			{
				state = UnitState.NORMAL;
				
			}
			
			if (bucket.scaleX >= 1) 
			{
				udpateCountdown("DONE!");
//				trace("Bomber ready!");
				callout("Water\nBomber\nFULL!",this,true,false);
				bucket.scaleX = 1;
			}
		}
		
		protected function waterDrop(passedTime:Number):void
		{
			if (bucket.scaleX > 0)
			{
				if (waterDropCounter % waterDropFrequency == 0) map.dropWater(this, Math.min(bucketWidth,Math.round(bucketWidth*bucket.scaleX*2)), 2);
				bucket.scaleX = bucket.scaleY = Math.max(0,bucket.scaleX - speedWaterDrop * passedTime*30 * TweenMax.globalTimeScale);
				waterDropCounter++;
			}
			else
			{
				waterDropCounter = 0;
				state = UnitState.NORMAL;
				if (pumpZone.numChildren != 0)
				{
					progress = 0;
					linePath.points = new Array();
					newLinePathPoints = new Array();
					updateSpeedAndProgress();
					linePath.appendPoint(new Point(x, y));
					calculatePoints(new Point(x, y), new Point(pumpZone.x, pumpZone.y), linePath);
					linePath.appendPoint(new Point(pumpZone.x, pumpZone.y));
					updateSpeedAndProgress();
					pumpZone.alpha = 1;
					targetPoint.x = pumpZone.x;
					targetPoint.y = pumpZone.y;
					targetPoint.rotation = pumpZone.rotation;
					sendUpdatedUnit(true);
				}
			}
		}		
		
		override public function sendUpdatedUnit(_forceSend:Boolean = false):void 
		{
			if (isMyUnit && (sendCounter >= 5 || _forceSend))
			{
				sendCounter = 0;
				var gameData:Object = new Object();
				gameData.action = GameDataType.UNIT_NEWLINEPATHPOINTS;
				gameData.userID = PlayerIOConnection.myUserID;
				gameData.points = linePath.points;
				gameData.speed = speed;
				gameData.speedOnPath = speedOnPath;
				gameData.progress = progress;
				//				data.isOffScreen = isOffScreen;
				//				data.isEnteringMap = isEnteringMap;
				gameData.id = id;
				gameData.type = type;
				gameData.oldLength = oldLength;
				gameData.oldProgress = oldProgress;
				gameData.x = x;
				gameData.y = y;
				gameData.rotation = unitIcon.rotation;
				gameData.targetPointRotation = targetPoint.rotation;
				gameData.pointName = pointName;
				gameData.playerName = PlayerIOConnection.playerName;
				gameData.bucketScaleX = bucket.scaleX;
				gameData.state = state;
				gameData.isClearPumpZoneEnabled = (pumpZone.numChildren > 0);
				gameData.isNewLinepath = isNewLinepath;				
				isNewLinepath = false;

				
				map.sendDataToPlayerIO(gameData);
			}
			sendCounter++;
		}
		
		override public function processUnit(_data:Object):void
		{
			//			if (map.isSetupComplete)
			if (true)
			{
				var tempPoints:Array = new Array();
				for (var i:int = 0; i < _data.points.length; i++)
				{
					tempPoints.push(new Point(_data.points[i].x, _data.points[i].y))
				}
				linePath.points = tempPoints;
				speed = _data.speed;
				speedOnPath = _data.speedOnPath;
				progress = _data.progress;
				oldLength = _data.oldLength;
				state = _data.state;
				oldProgress = _data.oldProgress;
				bucket.scaleX = bucket.scaleY = _data.bucketScaleX;
				if (redblox.isActive && _data.isNewLinepath) SoundAS.play("WaterBomberDropWater").fadeTo(0,7000);
				//				isEnteringMap = _data.isEnteringMap;
				//				isOffScreen = _data.isOffScreen;
				//linePath.points = tempPoints;
				//this.x = _data.x;
				//this.y = _data.y;
				unitIcon.rotation = _data.rotation;
				targetPoint.rotation = _data.targetPointRotation;
				if (linePath.points.length > 0)
				{
					targetPoint.x = linePath.points[linePath.points.length-1].x;
					targetPoint.y = linePath.points[linePath.points.length-1].y;
					map.lineLayer.addChild(targetPoint);
				}
				else
				{
					if (map.lineLayer.contains(targetPoint)) map.lineLayer.removeChild(targetPoint);
				}
				
				if (_data.isClearPumpZoneEnabled == false)
				{
					pumpZone.removeChildren();
					clearPumpZone.isEnabled = false;
				}
//				else
//				{
//					targetPoint.x = pumpZone.x;
//					targetPoint.y = pumpZone.y;
//					targetPoint.rotation = pumpZone.rotation;
//				}
				
				//this.img.icon.visible = _data.visible;
				udpatePlayerName(_data.playerName);
				userID = _data.userID;
				updateSpeedAndProgress();
				//map.lineLayer.addChild(targetPoint);
				showPointType(_data.pointName);
				updatePointTypeByTerrain();
				clearLine();
				this.x = _data.x;
				this.y = _data.y;
				drawLineCounter = 10;
				drawLine();
				isDragAndDrop = false;
				this.visible = true;
				if (map.unitLayer.contains(this) == false) map.unitLayer.addChild(this);
				
				//if (tempPoints.length > 0) TweenMax.to(endPoint, 0, { x:tempPoints[tempPoints.length - 1].x, y: tempPoints[tempPoints.length - 1].y, autoAlpha:1 } );
			}
			else
			{
				//				this.x = _data.x;
				//				this.y = _data.y;
				//				this.img.rotation = _data.rotation;
				//				this.img.icon.visible = _data.visible;
			}
		}
		
		override protected function onMouseMoveUnit():void
		{
			super.onMouseMoveUnit();
		}
		
		override protected function onMouseUpUnit():void
		{
			if (isDragAndDrop) 
			{
				callout("I can fill up \nOFF SCREEN or draw a \npath to DEEP WATER.", this, true, false);
				map.unitLayer.addChild(this);
			}
			super.onMouseUpUnit();
			if (bucket.scaleX > 0) linePath.removePointByIndex(linePath.points.length-1);
			
			clearPumpZone.isEnabled = (pumpZone.numChildren > 0) ? true : false; 			
			map.panelUnitMenu.show(containerMenu, this, "Bomber");
			
			if (targetPoint.contains(pointType["BomberPumpZone"]))
			{
				//trace("redblox.savedGameInfo.data.calloutBomberPumpZone: " + redblox.savedGameInfo.data.calloutBomberPumpZone);
				if (redblox.savedGameInfo.data.calloutBomberPumpZone == false)
				{
					callout("Rotate the bomber's \nPUMP ZONE to be completely \nover DEEP WATER for \nmaximum fill up.", targetPoint, false, false);
				}
			}
//			else if (targetPoint.contains(pointType["BomberDropZone"]))
//			{
//				if (redblox.savedGameInfo.data.calloutBomberDropZone == false)
//				{
//					callout("You can rotate your bomber's DROP ZONE for better accuracy.", targetPoint);
//				}
//			}

		}
		
		private function activatePointDirection():void
		{
			map.addEventListener(TouchEvent.TOUCH, onTouchMap);
		}
		
		private function onTouchMap(e:TouchEvent):void
		{
			var touch:Touch = e.getTouch(map);
			if (touch && touch.phase == TouchPhase.BEGAN)
			{
//				onMouseMoveTargetPoint();
				var point:Point = new Point((touch.globalX - map.x)/map.scaleX,(touch.globalY - map.y)/ map.scaleY);
				
				targetPoint.rotation = redblox.point_direction(targetPoint.x, targetPoint.y, point.x, point.y) * Math.PI/180;
				map.removeEventListener(TouchEvent.TOUCH, onTouchMap);

//				var point:Point = this.globalToLocal(new Point(touch.globalX, touch.globalY))
//				
//				targetPoint.rotation = redblox.point_direction(touch.globalX, touch.globalY, this.x, this.y);
			}
//			else if (touch.phase == TouchPhase.ENDED)
//			{
//			}
//			else if (touch.phase == TouchPhase.MOVED)
//			{
//				onMouseMoveTargetPoint();
//			}
		}
		
		override protected function onMouseDownUnit():void
		{
			if (redblox.isActive) SoundAS.play("WaterBomberDropWater").fadeTo(0,7000);
			super.onMouseDownUnit();
			state = UnitState.NORMAL;
		}
		override protected function onMouseDownTargetPoint():void
		{
//			if (!isDragAndDrop)
//			{
				//trace("\nBomber.onMouseMoveTargetPoint()");
				//isCollect = true;
				if (!targetPoint.contains(pointType["BomberPumpZone"]) && !targetPoint.contains(pointType["BomberDropZone"]))
				{
					isCollect = true;
					map.openMinimap();
				}
				if (targetPoint.contains(pointType["BomberPumpZone"]) || targetPoint.contains(pointType["BomberDropZone"]))
				{
					//activatePointDirection();
				}
				trace(" > isCollect: " + isCollect);
//			}
		}
		override protected function onMouseMoveTargetPoint():void
		{
//			if (!isDragAndDrop)
//			{
				trace("\nBomber.onMouseMoveTargetPoint()");
				
				if (!isCollect)
				{
					var point:Point = new Point((touchUnit.globalX - map.x)/map.scaleX,(touchUnit.globalY - map.y)/ map.scaleY);
					
					targetPoint.rotation = redblox.point_direction(targetPoint.x, targetPoint.y, point.x, point.y) * Math.PI/180;
				}
				trace(" > isCollect: " + isCollect);
//			}
		}
		
		override protected function onMouseUpTargetPoint():void
		{
			super.onMouseUpTargetPoint();
			if (bucket.scaleX > 0) linePath.removePointByIndex(linePath.points.length-1);
			
			clearPumpZone.isEnabled = (pumpZone.numChildren > 0) ? true : false; 			
			map.panelUnitMenu.show(containerMenu, this, "Bomber");
			
			if (targetPoint.contains(pointType["BomberPumpZone"]))
			{
				redblox.savedGameInfo.data.calloutBomberPumpZone = true;
				redblox.savedGameInfo.flush();
			}
			if (targetPoint.contains(pointType["BomberDropZone"]))
			{
				redblox.savedGameInfo.data.calloutBomberDropZone = true;
				redblox.savedGameInfo.flush();
			}
		}
		
		override protected function updateUnitOnNewTerrain():void
		{
			super.updateUnitOnNewTerrain();
			if (terrainUnderUnitPrevious != terrainUnderUnit)
			{
				isFireUnderUnit = map.checkToSeeIfThereIsFire(this, bucketWidth);
			}
		}
		
		override protected function clearLine():void
		{
			super.clearLine();
			
			if (dropWaterPointArray)
			{
				for (var i:int = 0; i < dropWaterPointArray.length; i++)
				{
					dropWaterPointArray[i].visible = false;
				}
			}			
			
		}
		
		
		override protected function drawLine():void
		{
			if (linePath.points.length > 0)
			{
//				drawLineCounter = 0;
				var points:Number = linePath.totalLength / gap;
				var num:Number = 1 / points;
				var angle:Number;
				
				//spread the points evenly  
				var p:Number = 0;
				var point:Point;
				var prevPoint:Point;
				
				var currentDot:int = 0;
				
				var upToPoints:Array = linePath.getAllPointsUpToProgress(progress);
				var afterPoints:Array = linePath.getAllPointsAfterProgress(progress);
				
				for (var i:int = 0; i < dropWaterPointArray.length; i++)
				{
					dropWaterPointArray[i].visible = false;
				}
				
				for (var i:int = 0; i < upToPoints.length; i++)
				{
					linePointArray[currentDot].visible = false;
					currentDot++;
				}
				
				var lastPoints:int;
				for (var i:int = 0; i < afterPoints.length; i++)
				{
					if (linePointArray[currentDot])
					{
						if (!linePointArray[currentDot].visible) 
						{
							linePointArray[currentDot].x = afterPoints[i].x - 8;
							linePointArray[currentDot].y = afterPoints[i].y - 8;
							linePointArray[currentDot].visible = true;
							linePointArray[currentDot].alpha = 1;
							if (!isMyUnit) TweenMax.to(linePointArray[currentDot], 1, {alpha:0.4, delay:0.02 * currentDot, overwrite:true});
							
						}
						currentDot++;
					}
					else
					{
						break;
					}
					
				}
				
				if (bucket.scaleX > 0)
				{
					currentDot = 0;
					var foundFire:Boolean = false;
					
					// First check if any dropWaterCircles would be over fire
					for (var i:int = afterPoints.length; i > afterPoints.length-Math.ceil(10*bucket.scaleX); i--)
					{
						if ( afterPoints[i] && map.checkToSeeIfThereIsFire(afterPoints[i], bucketWidth))
						{
							foundFire = true;
							break;
						}
					}
					if (foundFire) // if you find any fire along the end of path, show all dropCircles.
					{
						var everyOtherDot:int = 0;
						for (var i:int = afterPoints.length; i > afterPoints.length-Math.ceil(10*bucket.scaleX); i--)
						{
							if (afterPoints[i] && everyOtherDot % 2 == 0)
							{
								dropWaterPointArray[currentDot].x = afterPoints[i].x - 30;
								dropWaterPointArray[currentDot].y = afterPoints[i].y - 30;
								dropWaterPointArray[currentDot].visible = true;
								currentDot++;
							}
							everyOtherDot++;
						}
					}
					
					if (state != UnitState.WATER_DROP && !isCollect && afterPoints.length < Math.ceil(9*bucket.scaleX) && isFireUnderUnit)
					{
						if (redblox.isActive) SoundAS.play("WaterBomberDropWater").fadeTo(0,7000);
						state = UnitState.WATER_DROP;
					}
				}				
				
			}
//			drawLineCounter++;
		}
		
		override public function reset():void
		{
			super.reset();
			pumpZone.removeFromParent();
		}
		
		
	}
}