package units
{
	import com.greensock.TweenMax;
	import com.greensock.motionPaths.LinePath2D;
	
	import flash.geom.Point;
	import flash.utils.Dictionary;
	
	import connections.PlayerIOConnection;
	
	import maps.GameDataType;
	import maps.Map;
	import maps.TerrainState;
	import maps.TerrainType;
	
	import org.gestouch.events.GestureEvent;
	import org.gestouch.gestures.TapGesture;
	
	import playerio.generated.PlayerIO;
	
	import starling.animation.IAnimatable;
	import starling.core.Starling;
	import starling.display.Image;
	import starling.display.MovieClip;
	import starling.display.Sprite;
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.filters.BlurFilter;
	import starling.filters.ColorMatrixFilter;
	
	import treefortress.sound.SoundAS;
	
	public class FireBoat extends Unit
	{
		public var wetTrees:Array = new Array();
		public var waterIntensity:Number = 2;
		public var waterRadius:Number = 300;
		public var waterRadiusWhenStopped:Number = 2;
		public var waterRadiusWhenMoving:Number = 1.5;
		public var hoseArray:Array = new Array();
		public var currentHoseLength:Number = 0;
		public var isMovingToWaterSource:Boolean = false;
		public var isWaterOn:Boolean = true;
		public var isUnitOnFireHydrant:Boolean = false;
		public var waterPoint:Point;
		private var tappedOnce:Boolean = false;
		public var lastGoodTerrain:Point;
		
		public function FireBoat(_map:Map, _isMyUnit:Boolean, _id:int)
		{
			type = UnitType.FIRE_BOAT;
			super(_map, _isMyUnit, _id);
		}
		
		override public function toggleNight():void
		{
			super.initializeUnitIcon((map.isNight) ? "FireBoatNight" : "FireBoat", false);
		}
		
		override public function initializeUnit():void
		{
			super.initializeUnit();
			
			callout("Place me in \nDEEP WATER", this, true, false);
		}

		override protected function initializeOnce():void
		{
			// Add dots to use for line path
			linePointArray = new Array();
			for (var i:int = 0; i < numDots; i++)
			{
				linePointArray.push(new Image(Assets.getAtlas().getTexture("dot")));
				map.lineLayer.addChild(linePointArray[i]);
			}
			
			speed = 1;
			state = UnitState.SPRAY;
			
			super.initializeUnitIcon((map.isNight) ? "FireBoatNight" : "FireBoat", false);
			addNewPointType("targetNormal");
			showPointType("targetNormal");
			
			unitIcon.currentFrame = 0;
			
			//var dropShadow:BlurFilter = BlurFilter.createDropShadow(5);
			
			//this.filter = dropShadow;
//			if (isMyUnit)
//			{
//				tapGesture = new TapGesture(unitIcon);
//				tapGesture.numTouchesRequired = 1;
//			}
			
			unitIconShadow.x = 3;
			unitIconShadow.y = 3;
			
			rotateUnitAlongPath();
		}
		
		override protected function onDoubleTapUnitIcon(event:GestureEvent):void
		{
			trace("Engine.onTapUnitIcon");
			if (tappedOnce)
			{
				if (state == UnitState.NORMAL) 
				{
					state = UnitState.SPRAY;
					unitIcon.currentFrame = 0;
					sprayWater();
				}
				else
				{
					state = UnitState.NORMAL;
					unitIcon.currentFrame = 1;
					stopWater();
				}
				sendUpdatedUnit(true);
			}
			tappedOnce = true;
		}
		
		override public function disable(_removeJuggler:Boolean = true):void
		{
			super.disable(_removeJuggler);
			if (_removeJuggler) stopWater();
			
		}
		
		
//		override public function initializeUnit():void
//		{
//			super.initializeUnit();
//			
////			if (isMyUnit)
////			{
////				tapGesture.addEventListener(GestureEvent.GESTURE_RECOGNIZED, onDoubleTapUnitIcon);
////			}
//		}
		
		override public function reset():void
		{
			super.reset();
//			if (isMyUnit) tapGesture.removeEventListener(GestureEvent.GESTURE_RECOGNIZED, onDoubleTapUnitIcon);
			stopWater();
			state = UnitState.SPRAY;
		}
		
		override protected function updatePointTypeByTerrain():void
		{
//			try
//			{				
//				var terrainType:Number = map.terrain[Math.round(targetPoint.x/map.blitSize)][Math.round(targetPoint.y/map.blitSize)].type;
//				if (terrainType == TerrainType.WATER_DEEP)
//				{
//					showPointType("targetHelicopterWaterPump");
//				}
//				else
//				{
//					showPointType("targetNormal");
//				}
//			}
//			catch(e:Error)
//			{
				showPointType("targetNormal");
//			}
		}
		
		override public function advanceTime(passedTime:Number):void
		{
			rotateUnitAlongPath();
			
			if (isDragAndDrop)
			{
				return;
			}
			
			if (isCollect) 
			{
				// Add new linePath points
				var point:Point = new Point((touchUnit.globalX - map.x)/map.scaleX,(touchUnit.globalY - map.y)/ map.scaleY);
				targetPoint.x = point.x;
				targetPoint.y = point.y;
				updatePointTypeByTerrain();
				
				map.updateMinimap(targetPoint);
				
				var lastPoint:Point = linePath.points[linePath.points.length-1];
				
				if (lastPoint && Point.distance(point, lastPoint) >= maxDistance)
				{
					calculatePoints(lastPoint, point, linePath);
					updateSpeedAndProgress();
				}
				targetPoint.rotation = linePath.getAngleAtProgress(1);
				sendUpdatedUnit();
			}
			
			updateUnitOnNewTerrain();
			
			// Follow linePath if it exists
			if (linePath.totalLength > 0)
			{
				// Draw series of dots along unit's linePath
				drawLine();
				
				// Advance unit's progress along linePath
				progress += speedOnPath * TweenMax.globalTimeScale * passedTime*30 * speedVariantBasedOnTerrain;
				if (progress > 1) progress = 1;
				if (linePath.points.length > 1) 
				{
					linePath.renderObjectAt(this, progress, false, 0);
					targetRotation = linePath.angle;
				}
				
				oldProgress = progress;
				
				if (progress == 1) arrivedAtDestination();
			}
			
			//this.scaleX = this.scaleY = Math.min(1, 1/map.scaleX);

		}
		
		/**
		 *
		 */
		public function stopWater():void
		{
			for (var i:int = 0; i < wetTrees.length; i++)
			{
				if (wetTrees[i])
				{
					wetTrees[i].waterCount--;
					map.blitIt(wetTrees[i], false);
				}
			}
			
			wetTrees = new Array();
		}
		
		/**
		 *
		 */
		public function sprayWater():void
		{
			stopWater();
			
			//trace("spray water");
			var i, j:Number;
			var _terrain:Object;
			var locX:int = Math.floor(this.x / map.blitSize);
			var locY:int = Math.floor(this.y / map.blitSize);
			
//			if (terrainUnderUnit)
//			{
//				if (terrainUnderUnit.type == TerrainType.TREE)
//				{
//					waterRadius = 40;
//				}
//				else
//				{
//					waterRadius = 55;
//				}
//			}
			var spread:Number = Math.ceil(waterRadius / map.blitSize / 2);
			
			for (i = -spread; i < spread; i++)
			{
				for (j = -spread; j < spread; j++)
				{
					try
					{
						_terrain = map.terrain[locX + i][locY + j];
						if (Point.distance(new Point(this.x, this.y), new Point(this.x + map.blitSize * i, this.y + map.blitSize * j)) < waterRadius * map.mapScale / 2)
						{
							wetTrees.push(_terrain);
							_terrain.waterCount++;
							_terrain.wetMultiplier = (1 - (waterIntensity * map.waterResistanceList[_terrain.type]))/_terrain.waterCount;
							//if (_terrain.state != "")
							map.blitIt(_terrain, false);
						}
					}
					catch (err:Error)
					{
					}
				}
			}
		}
		
		
		override protected function updateUnitOnNewTerrain():void
		{
			super.updateUnitOnNewTerrain();
			
			//speed = UnitSpeeds[state][terrainUnderUnit.type][terrainUnderUnit.state]
			
			if (terrainUnderUnit && terrainUnderUnit != terrainUnderUnitPrevious)
			{
				
				if (state == UnitState.NORMAL)
				{
					if (terrainUnderUnit == null)
					{
						speedVariantBasedOnTerrain = 10;
					}
					else if (terrainUnderUnit.state == TerrainState.FIRE || terrainUnderUnit.state == TerrainState.FIRE_WET)
						speedVariantBasedOnTerrain = 0;
					else if (terrainUnderUnit.state == TerrainState.DEAD || terrainUnderUnit.state == TerrainState.DEAD_WET)
						speedVariantBasedOnTerrain = 0.5;
					else if (terrainUnderUnit.state == TerrainState.BURNT || terrainUnderUnit.state == TerrainState.BURNT_WET)
						speedVariantBasedOnTerrain = 0.8;
					else if (terrainUnderUnit.state == TerrainState.CUT || terrainUnderUnit.state == TerrainState.CUT_WET)
						speedVariantBasedOnTerrain = 1;
					else if (terrainUnderUnit.type == TerrainType.WATER_DEEP)
						speedVariantBasedOnTerrain = 0.05;
					else if (terrainUnderUnit.type == TerrainType.WATER_SHALLOW)
						speedVariantBasedOnTerrain = 0.75;
					else if (terrainUnderUnit.type == TerrainType.SAND)
						speedVariantBasedOnTerrain = 1.25;
					else if (terrainUnderUnit.type == TerrainType.GRAVEL)
						speedVariantBasedOnTerrain = 1.5;
					else if (terrainUnderUnit.type == TerrainType.ROAD)
						speedVariantBasedOnTerrain = 1.5;
					else if (terrainUnderUnit.type == TerrainType.GRASS)
						speedVariantBasedOnTerrain = 1;
					else if (terrainUnderUnit.type == TerrainType.BUSH)
						speedVariantBasedOnTerrain = 0.5; //0.80;
					else if (terrainUnderUnit.type == TerrainType.TREE)
						speedVariantBasedOnTerrain = 0.25; // 0.5;
					else if (terrainUnderUnit.type == TerrainType.MARSH)
						speedVariantBasedOnTerrain = 0.3;
					else
						speedVariantBasedOnTerrain = 1;
				}
				else if (state == UnitState.SPRAY)
				{
//					img.gotoAndStop(1);
					//sprayWater();
					if (terrainUnderUnit && terrainUnderUnit != terrainUnderUnitPrevious) 
						unitIcon.currentFrame = 0;
					
					if (terrainUnderUnit == null)
					{
						speedVariantBasedOnTerrain = 3;
					}
					else if (isTerrainNearby(TerrainType.WATER_DEEP,1))
					{
						speedVariantBasedOnTerrain = 1;
						lastGoodTerrain = new Point(x,y);
						if (linePath.points.length > 1) stopWater();
					}
					else if (isTerrainArrayNearby([TerrainType.TREE, TerrainType.BUSH, TerrainType.GRASS, TerrainType.ROAD, TerrainType.GRAVEL, TerrainType.MARSH, TerrainType.SAND, TerrainType.LAWN],4))
					{
						callout("I can not \ngo further!", this, true);
						arrivedAtDestination();
						if (lastGoodTerrain)
						{
							x = lastGoodTerrain.x;
							y = lastGoodTerrain.y;
						}
					}
					else if (isTerrainNearby(TerrainType.WATER_SHALLOW,1))
					{
						speedVariantBasedOnTerrain = 0.4;
						lastGoodTerrain = new Point(x,y);
						if (linePath.points.length > 1) stopWater();
					}
					else
					{
						callout("I can not \ngo further!", this, true);
						arrivedAtDestination();
						if (lastGoodTerrain)
						{
							x = lastGoodTerrain.x;
							y = lastGoodTerrain.y;
						}
						//speedVariantBasedOnTerrain = 0;
					}

				}
			}
		}
		
		/**
		 * 
		 * @param	unit
		 */
		override protected function arrivedAtDestination ():void
		{
			//trace("\nUnit.arrivedAtDestination();");
			clearLine();
			progress = 0;
			linePath.points = new Array();
			map.lineLayer.removeChild(targetPoint);
			sprayWater();
			if (redblox.isActive) SoundAS.play("HoseTeam");
			callout("Fire Board \narrived!", this, true, false);

		} 
		
		
		/**
		 * 
		 * @param	_terrain
		 */
		override public function sendUpdatedUnit(_forceSend:Boolean = false):void 
		{
			if (isMyUnit && (sendCounter >= 5 || _forceSend))
			{
				sendCounter = 0;
				var gameData:Object = new Object();
				gameData.action = GameDataType.UNIT_NEWLINEPATHPOINTS;
				gameData.userID = PlayerIOConnection.myUserID;
				
				gameData.points = linePath.points;
				gameData.speed = speed;
				gameData.speedOnPath = speedOnPath;
				gameData.progress = progress;
				gameData.state = state;
				gameData.speedVariantBasedOnTerrain = speedVariantBasedOnTerrain;
				//				data.isOffScreen = isOffScreen;
				//				data.isEnteringMap = isEnteringMap;
				gameData.id = id;
				gameData.type = type;
				gameData.oldLength = oldLength;
				gameData.oldProgress = oldProgress;
				gameData.x = x;
				gameData.y = y;
				gameData.rotation = unitIcon.rotation;
				gameData.targetPointRotation = targetPoint.rotation;
				gameData.pointName = pointName;
				gameData.playerName = PlayerIOConnection.playerName;
				gameData.isNewLinepath = isNewLinepath;				
				isNewLinepath = false;
				
				map.sendDataToPlayerIO(gameData);
			}
			sendCounter++;
		}
		
		override public function processUnit(_data:Object):void
		{
			//			if (map.isSetupComplete)
			if (true)
			{
				var tempPoints:Array = new Array();
				for (var i:int = 0; i < _data.points.length; i++)
				{
					tempPoints.push(new Point(_data.points[i].x, _data.points[i].y))
				}
				linePath.points = tempPoints;
				speed = _data.speed;
				speedOnPath = _data.speedOnPath;
				progress = _data.progress;
				oldLength = _data.oldLength;
				oldProgress = _data.oldProgress;
				unitIcon.rotation = _data.rotation;
				targetPoint.rotation = _data.targetPointRotation;
				speedVariantBasedOnTerrain = _data.speedVariantBasedOnTerrain;
				state = _data.state;
				
				if (redblox.isActive && _data.isNewLinepath) SoundAS.play("FireBoat");
					
					
				if (state == UnitState.NORMAL) 
				{
					unitIcon.currentFrame = 1;
				}
				else
				{
					unitIcon.currentFrame = 0;
				}

				if (linePath.points.length > 0)
				{
					targetPoint.x = linePath.points[linePath.points.length-1].x;
					targetPoint.y = linePath.points[linePath.points.length-1].y;
					map.lineLayer.addChild(targetPoint);
				}
				else
				{
					if (map.lineLayer.contains(targetPoint)) map.lineLayer.removeChild(targetPoint);
				}
				//this.img.icon.visible = _data.visible;
				udpatePlayerName(_data.playerName);
				userID = _data.userID;
				updateSpeedAndProgress();
				//map.lineLayer.addChild(targetPoint);
				showPointType(_data.pointName);
				updatePointTypeByTerrain();
				clearLine();
				this.x = _data.x;
				this.y = _data.y;
				drawLineCounter = 10;
				drawLine();
				isDragAndDrop = false;
				this.visible = true;
				if (map.unitLayer.contains(this) == false) map.unitLayer.addChildAt(this, 0);
				
				if (targetUnit) // check to see if a helicopter is coming to pick him up
				{
					targetUnit.disable(false);
					targetUnit.targetUnit = null;
					targetUnit = null;
				}
				//if (tempPoints.length > 0) TweenMax.to(endPoint, 0, { x:tempPoints[tempPoints.length - 1].x, y: tempPoints[tempPoints.length - 1].y, autoAlpha:1 } );
			}
		}
		
		override protected function onMouseDownUnit():void
		{
			super.onMouseDownUnit();
			if (redblox.isActive) SoundAS.play("FireBoat");

			stopWater();
			speedVariantBasedOnTerrain = 1;
		}
		
		override protected function onMouseUpUnit():void
		{
			if (isDragAndDrop)
			{
				if (isTerrainNearby(TerrainType.WATER_DEEP,2))
				{
					callout("Ready to spray water! \nI'm a beast along the coast!", this, true, false);
					map.unitLayer.addChildAt(this,0);
					isDragAndDrop = false;
					arrivedAtDestination();
				}
				else
				{
					callout("I'm too far \nfrom DEEP WATER, place \nme somewhere else!", this, true, false);
					return;
				}
			}
			
			super.onMouseUpUnit();
		}
		
		override protected function onMouseMoveUnit():void
		{
			tappedOnce = false;
			super.onMouseMoveUnit();
		}
		
		
		override public function dispose():void
		{
			
			super.dispose();
		}
	}
}