package units
{
	import com.greensock.TweenMax;
	import com.greensock.motionPaths.LinePath2D;
	
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.geom.Point;
	import flash.utils.ByteArray;
	
	import connections.PlayerIOConnection;
	
	import feathers.controls.Button;
	
	import maps.GameDataType;
	import maps.Map;
	import maps.TerrainState;
	import maps.TerrainType;
	
	import starling.animation.IAnimatable;
	import starling.core.Starling;
	import starling.display.Image;
	import starling.display.MovieClip;
	import starling.display.Sprite;
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.filters.BlurFilter;
	import starling.textures.Texture;
	import starling.utils.Line;
	
	import treefortress.sound.SoundAS;
	
	public class Helicopter extends Unit
	{
		protected var isPumpingWater:Boolean;
		protected var bucket:Image;
		protected var bucketWidth:Number = 50;
		protected var speedWaterPump:Number = 1/30/2;		// 1 iteration / 30 fps / 3 sec = incremental delta of pumping water for 3 secs
		protected var speedWaterDrop:Number = 1/30/0.5;		// 1 iteration / 30 fps / 0.5sec = decremental delta of dropping water over 0.5;
		protected var waterDropCounter:int = 0; 			// incremental counter used with "waterDropFrequency"
		protected var waterDropFrequency:int = 30;			// to drop water 1 per second, make it 30 since there's 30fps
		private var dropWaterButton:Button;
		
		
		
		public function Helicopter( _map:Map, _isMyUnit:Boolean, _id:int)
		{
			type = UnitType.HELICOPTER;
			super(_map, _isMyUnit, _id);
		}
		
		override public function toggleNight():void
		{
			var currentScale:Number = 1;
			if (bucket) 
			{
				currentScale = bucket.scaleX;
				bucket.removeFromParent(true);
			}
			bucket = new Image(Assets.getAtlas().getTexture((map.isNight) ? "BucketBomberNight":"BucketBomber"));
			bucket.alignPivot();
			bucket.scaleX = bucket.scaleY = currentScale;
			
			super.initializeUnitIcon((map.isNight) ? "helicopterNight" : "helicopter", true, 15);
			
			iconUnitContainer.addChild(unitIconShadow);			
			iconUnitContainer.addChild(bucket);
			addChild(unitIconCollision);
			iconUnitContainer.addChild(unitIcon);
		}
		
		override protected function initializeOnce():void
		{
			super.initializeOnce();
			
			bucket = new Image(Assets.getAtlas().getTexture("BucketBomber"));
			bucket.alignPivot();
			
			super.initializeUnitIcon((map.isNight) ? "helicopterNight" : "helicopter", true, 15);
			
			addNewPointType("HelicopterPumpZone", true, 20);
			addNewPointType("HelicopterDropZone", true, 20);
			addNewPointType("HelicopterPickup", true, 20);
			
//			var dropShadow:BlurFilter = BlurFilter.createDropShadow(20);
			
			dropWaterButton = new Button();
			dropWaterButton.label = "Drop Water";
			dropWaterButton.addEventListener(starling.events.Event.TRIGGERED, onButtonClicked);
			
			containerMenu.addChild(dropWaterButton);
			
			speed = 1.2;
			
			//this.filter = dropShadow;
		}
		
		override public function initializeUnit():void
		{
			super.initializeUnit();
			
			callout("Place me anywhere", this, true, false);
			iconUnitContainer.addChild(unitIconShadow);			
			iconUnitContainer.addChild(bucket);
			addChild(unitIconCollision);
			iconUnitContainer.addChild(unitIcon);
			bucket.scaleX = bucket.scaleY = 0;
		}
		
		override protected function updatePointTypeByTerrain():void
		{
			try
			{
				if (targetUnit == null)
				{
					var terrainType:Number = map.terrain[Math.round(targetPoint.x/map.blitSize)][Math.round(targetPoint.y/map.blitSize)].type;
					if (terrainType == TerrainType.WATER_DEEP && bucket.scaleX < 1)
					{
						showPointType("HelicopterPumpZone");
					}
					else if (map.checkToSeeIfThereIsFire(targetPoint, bucketWidth*bucket.scaleX))
					{
						showPointType("HelicopterDropZone");
					}
					else
					{
						showPointType("targetNormal");
					}
				}
			}
			catch(e:Error)
			{
				showPointType("targetNormal");
			}
		}
		
		/**
		 * 
		 * @param	unit
		 */
		override protected function arrivedAtDestination ():void
		{
			super.arrivedAtDestination();
			
			trace("Helicopter.arrivedAtDestination();");
			trace("> state: " + state);
			//if (state == "PickupUnit" && targetAltitude == mapRoom.groundElevation && Point.distance(new Point(targetUnit.x, targetUnit.y), new Point(this.x, this.y)) < 5 * mapLayer.mapScale)
			if (targetUnit && !targetUnit.isOnHelicopter)// && Point.distance(new Point(targetUnit.x, targetUnit.y), new Point(this.x, this.y)) < 5 * mapLayer.mapScale)
			{
				if (terrainUnderUnit.type == TerrainType.TREE && terrainUnderUnit.state == TerrainState.NORMAL)
				{
					callout("Helicopters can't \npick up a unit in \ndense trees! Find \na safe clearing.", this, true);
				}
				else
				{
					trace("Picking up unit: ");
					targetUnit.x = -35;
					targetUnit.y = 0;
					this.addChild(targetUnit);
					targetUnit.scaleX = targetUnit.scaleY = 0.5;
					targetUnit.isOnHelicopter = true;
					targetUnit.disable();
					targetUnit.unitIcon.currentFrame = 0;
	
	
					showPointType("targetNormal");
				}
			}
			else if (targetUnit && targetUnit.isOnHelicopter)//state == UnitState.WATER_DROP)
			{
				trace("Dropping unit");
				if (terrainUnderUnit.type == TerrainType.TREE && terrainUnderUnit.state == TerrainState.NORMAL)
				{
					callout("Helicopters can't \ndrop a unit on \ndense trees! Find \na safe clearing.", this, true);
				}
				else
				{
					state = UnitState.NORMAL;
					
					map.unitLayer.addChildAt(targetUnit,0);
					targetUnit.x = x;
					targetUnit.y = y;
					
					//if (targetUnit.isMyUnit) 
					targetUnit.scaleX = targetUnit.scaleY = 1;
					//else targetUnit.scaleX = targetUnit.scaleY = 0.6;
					
					targetUnit.targetUnit = null;
					targetUnit.isOnHelicopter = false;
					
					targetUnit.enable();
					targetUnit = null;
					
				}
			}
			else if (map.checkToSeeIfThereIsFire(this, bucketWidth) && bucket.scaleX > 0)
			{
				trace("> there is fire here");
				state = UnitState.WATER_DROP;
				
				if (redblox.isActive) SoundAS.play("HelicopterDropWater");
			}
			else if (map.terrainTypeUnderPoint(this, TerrainType.WATER_DEEP))
			{
				trace("> there is water here");
				state = UnitState.WATER_PUMP;
				
				if (redblox.isActive) SoundAS.play("HelicopterPumpWater");
			}
		} 
		
		override public function advanceTime(passedTime:Number):void
		{
			super.advanceTime(passedTime);
			super.rotateUnitAlongPath();
						
			if (isDragAndDrop || state == UnitState.DEAD)
			{
				return;
			}
			
			if (terrainUnderUnit == null)
			{
				state = UnitState.OFFSCREEN;
				if (terrainUnderUnitPrevious != null) 
					callout("Off-Screen PUMP ZONE \ntakes 40 seconds to \nfill up.", this, true);
			}
			else if (terrainUnderUnit && state == UnitState.OFFSCREEN)
			{
				state = UnitState.NORMAL;
				if (this.contains(countdownPopup)) removeChild(countdownPopup);
			}
			
			if (targetUnit && targetUnit.isOnHelicopter)
			{
				targetUnit.x = -35 * Math.cos(unitIcon.rotation);
				targetUnit.y = -35 * Math.sin(unitIcon.rotation);
			}
			
			if (isCollect) checkIfPickupUnit();
						
			if (state == UnitState.WATER_PUMP)
			{
//				trace("Helicopter state: WATER_PUMP");
				waterPump(passedTime);
			}
			else if (state == UnitState.OFFSCREEN)
			{
//				trace("Helicopter state: OFFSCREEN");
				if (!this.contains(countdownPopup)) addChild(countdownPopup);
				waterPump(passedTime);
				
			}
			else if (state == UnitState.WATER_DROP)
			{
				waterDrop(passedTime);
			}
		}
		
		protected function checkIfPickupUnit():void
		{
			if (targetUnit && targetUnit.isOnHelicopter) return;
			
			var isUnitAvailableForPickup:Boolean = false;
			var _targetUnit:Unit;
			//targetUnit = null;
			
			if (bucket.scaleX == 0) // if helicopter has no water, then check for units to pickup
			{
				var cutTeamUnit:CutTeam;
				for (var unitID in map.unitPool[UnitType.CUT_TEAM])
				{
					cutTeamUnit = map.unitPool[UnitType.CUT_TEAM][unitID];
					
					if (cutTeamUnit.parent && Point.distance(new Point(targetPoint.x,targetPoint.y), new Point(cutTeamUnit.x, cutTeamUnit.y)) < 25)
					{
						_targetUnit = cutTeamUnit;
						isUnitAvailableForPickup = true;
						
						
						break;
					}
				}
				
				var hoseTeamUnit:HoseTeam;
				for (var unitID in map.unitPool[UnitType.HOSE_TEAM])
				{
					hoseTeamUnit = map.unitPool[UnitType.HOSE_TEAM][unitID];
					
					if (hoseTeamUnit.parent && Point.distance(new Point(targetPoint.x,targetPoint.y), new Point(hoseTeamUnit.x, hoseTeamUnit.y)) < 25)
					{
						_targetUnit = hoseTeamUnit;
						isUnitAvailableForPickup = true;
						
						
						break;
					}
				}

				var civilianUnit:Civilian;
				for (var unitID in map.unitPool[UnitType.CIVILIAN])
				{
					civilianUnit = map.unitPool[UnitType.CIVILIAN][unitID];
					
					if (civilianUnit.parent && Point.distance(new Point(targetPoint.x,targetPoint.y), new Point(civilianUnit.x, civilianUnit.y)) < 25)
					{
						_targetUnit = civilianUnit;
						isUnitAvailableForPickup = true;
						
						
						break;
					}
				}
			}
			
			if (isUnitAvailableForPickup)
			{
				if (targetUnit != _targetUnit) 
				{
					targetUnit = _targetUnit;
					showPointType("HelicopterPickup");
					trace("Pickup");
				}
			}
			else
			{
				if (targetUnit && !targetUnit.isOnHelicopter)
				{
					showPointType("targetNormal");
					targetUnit.targetUnit = null;
					targetUnit = null;
				}
				
			}
			
			//return isUnitAvailableForPickup;
		}
		
		protected function pickupUnit():void
		{
			// TODO Auto Generated method stub
			
		}
		
		protected function waterPump(passedTime:Number):void
		{
			//trace("waterPump()");
			if (map.terrainTypeUnderPoint(this, TerrainType.WATER_DEEP) && bucket.scaleX < 1)
			{
				bucket.scaleX = bucket.scaleY = bucket.scaleX + speedWaterPump * passedTime * 30 * TweenMax.globalTimeScale;
			}
			else if (state == UnitState.OFFSCREEN && bucket.scaleX < 1)
			{
				bucket.scaleX = bucket.scaleY = bucket.scaleX + (speedWaterPump/20) * passedTime * 30 * TweenMax.globalTimeScale;
				udpateCountdown((40 - Math.round(bucket.scaleX * 40)) + "s to FULL");
			}
			else if (state == UnitState.OFFSCREEN)
			{
				udpateCountdown("DONE!");
				// don't change state
			}
			else
			{
				state = UnitState.NORMAL;
			}
			
			if (bucket.scaleX > 1) bucket.scaleX = 1;
			
			dropWaterButton.isEnabled = true;

		}
		
		private function onButtonClicked(e:starling.events.Event):void
		{
			var buttonClicked:Button = e.target as Button;
			
			if (buttonClicked == dropWaterButton)
			{
				state = UnitState.WATER_DROP;
			}
		}
		
		protected function waterDrop(passedTime:Number):void
		{
			if (bucket.scaleX > 0)
			{
				if (waterDropCounter % waterDropFrequency == 0) map.dropWater(this, Math.min(bucketWidth,Math.round(bucketWidth*bucket.scaleX*2)), 2);
				bucket.scaleX = bucket.scaleY = bucket.scaleX - speedWaterDrop * passedTime*30 * TweenMax.globalTimeScale;
				waterDropCounter++;
			}
			else
			{
				bucket.scaleX = bucket.scaleY = 0;
				waterDropCounter = 0;
				state = UnitState.NORMAL;
			}
			dropWaterButton.isEnabled = false;
		}		
		
		override public function sendUpdatedUnit(_forceSend:Boolean = false):void 
		{
			if (isMyUnit && (sendCounter >= 5 || _forceSend))
			{
				sendCounter = 0;
				var gameData:Object = new Object();
				gameData.action = GameDataType.UNIT_NEWLINEPATHPOINTS;
				gameData.userID = PlayerIOConnection.myUserID;
				gameData.points = linePath.points;
				gameData.speed = speed;
				gameData.speedOnPath = speedOnPath;
				gameData.progress = progress;
				//				gameData.isOffScreen = isOffScreen;
				//				gameData.isEnteringMap = isEnteringMap;
				gameData.id = id;
				gameData.type = type;
				gameData.oldLength = oldLength;
				gameData.oldProgress = oldProgress;
				gameData.x = x;
				gameData.y = y;
				gameData.rotation = unitIcon.rotation;
				gameData.targetPointRotation = targetPoint.rotation;
				gameData.pointName = pointName;
				gameData.playerName = PlayerIOConnection.playerName;
				gameData.bucketScaleX = bucket.scaleX;
				gameData.isNewLinepath = isNewLinepath;				
				isNewLinepath = false;
				
				map.sendDataToPlayerIO(gameData);
			}
			sendCounter++;
		}
		
		override public function processUnit(_data:Object):void
		{
			//			if (map.isSetupComplete)
			if (true)
			{
				var tempPoints:Array = new Array();
				for (var i:int = 0; i < _data.points.length; i++)
				{
					tempPoints.push(new Point(_data.points[i].x, _data.points[i].y))
				}
				linePath.points = tempPoints;
				speed = _data.speed;
				speedOnPath = _data.speedOnPath;
				progress = _data.progress;
				oldLength = _data.oldLength;
				oldProgress = _data.oldProgress;
				bucket.scaleX = bucket.scaleY = _data.bucketScaleX;
				
				if (redblox.isActive && _data.isNewLinepath) SoundAS.play("HelicopterFlying");
					
					
				//				isEnteringMap = _data.isEnteringMap;
				//				isOffScreen = _data.isOffScreen;
				//linePath.points = tempPoints;
				//this.x = _data.x;
				//this.y = _data.y;
				unitIcon.rotation = _data.rotation;
				targetPoint.rotation = _data.targetPointRotation;
				if (linePath.points.length > 0)
				{
					targetPoint.x = linePath.points[linePath.points.length-1].x;
					targetPoint.y = linePath.points[linePath.points.length-1].y;
					map.lineLayer.addChild(targetPoint);
				}
				else
				{
					if (map.lineLayer.contains(targetPoint)) map.lineLayer.removeChild(targetPoint);
				}
				udpatePlayerName(_data.playerName);
				userID = _data.userID;
				//this.img.icon.visible = _data.visible;
				updateSpeedAndProgress();
				//map.lineLayer.addChild(targetPoint);
				showPointType(_data.pointName);
				updatePointTypeByTerrain();
				clearLine();
				this.x = _data.x;
				this.y = _data.y;
				drawLineCounter = 10;
				drawLine();
				isDragAndDrop = false;
				this.visible = true;
				if (map.unitLayer.contains(this) == false) map.unitLayer.addChild(this);

				checkIfPickupUnit();
				//if (tempPoints.length > 0) TweenMax.to(endPoint, 0, { x:tempPoints[tempPoints.length - 1].x, y: tempPoints[tempPoints.length - 1].y, autoAlpha:1 } );
				if (targetUnit && !targetUnit.isOnHelicopter)
				{
					targetPoint.x = targetUnit.x;
					targetPoint.y = targetUnit.y;
					targetUnit.disable(false);
					targetUnit.targetUnit = this;
					
				}
			}
			else
			{
				//				this.x = _data.x;
				//				this.y = _data.y;
				//				this.img.rotation = _data.rotation;
				//				this.img.icon.visible = _data.visible;
			}
		}
				
		override protected function onMouseMoveUnit():void
		{
			super.onMouseMoveUnit();
		}
		
		override protected function onMouseUpUnit():void
		{
			if (targetUnit && !targetUnit.isOnHelicopter)
			{
				targetPoint.x = targetUnit.x;
				targetPoint.y = targetUnit.y;
				targetUnit.disable(false);
				targetUnit.targetUnit = this;
				
			}
			if (isDragAndDrop) 
			{
				callout("Helicopters are \nmulti-purpose. They can \ntransport units and \ncan carry water.", this, true, false);
				map.unitLayer.addChild(this);
			}
			super.onMouseUpUnit();
			dropWaterButton.isEnabled = (bucket.scaleX > 0) ? true : false; 
			
			map.panelUnitMenu.show(containerMenu, this, "Helicopter");
		}
		
		override protected function onMouseUpTargetPoint():void
		{
			super.onMouseUpTargetPoint();
			dropWaterButton.isEnabled = (bucket.scaleX > 0) ? true : false; 
			map.panelUnitMenu.show(containerMenu, this, "Helicopter");
		}
		
		override protected function onMouseDownUnit():void
		{
			
			if (redblox.isActive) SoundAS.play("HelicopterFlying");
			super.onMouseDownUnit();
			state = UnitState.NORMAL;
		}
	}
}