package units
{
	import com.greensock.TweenMax;
	import com.greensock.motionPaths.LinePath2D;
	
	import flash.geom.Point;
	import flash.utils.Dictionary;
	
	import connections.PlayerIOConnection;
	
	import maps.GameDataType;
	import maps.Map;
	import maps.TerrainState;
	import maps.TerrainType;
	
	import org.gestouch.events.GestureEvent;
	import org.gestouch.gestures.TapGesture;
	
	import starling.animation.IAnimatable;
	import starling.core.Starling;
	import starling.display.Image;
	import starling.display.MovieClip;
	import starling.display.Sprite;
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.filters.BlurFilter;
	import starling.filters.ColorMatrixFilter;
	
	import treefortress.sound.SoundAS;
	
	public class Loschpanzer extends Unit
	{
		public var wetTrees:Array = new Array();
		public var waterIntensity:Number = 1.1;
		public var waterRadius:Number = 60;
		public var waterRadiusWhenStopped:Number = 2;
		public var waterRadiusWhenMoving:Number = 1.5;
		public var hoseArray:Array = new Array();
		public var currentHoseLength:Number = 0;
		public var isMovingToWaterSource:Boolean = false;
		public var isWaterOn:Boolean = true;
		public var isUnitOnFireHydrant:Boolean = false;
		public var waterPoint:Point;
		private var tappedOnce:Boolean = false;
		public var lastGoodTerrain:Point;
		
		public function Loschpanzer(_map:Map, _isMyUnit:Boolean, _id:int)
		{
			type = UnitType.LOSCHPANZER;
			super(_map, _isMyUnit, _id);
		}
		
		override public function toggleNight():void
		{
			super.initializeUnitIcon((map.isNight) ? "LoschpanzerNight" : "Loschpanzer", true, 1);
		}
		
		override public function initializeUnit():void
		{
			super.initializeUnit();
			
			callout("Place me on a \nROAD or GRAVEL", this, true, false);
		}
		
		override protected function initializeOnce():void
		{
			// Add dots to use for line path
			linePointArray = new Array();
			for (var i:int = 0; i < numDots; i++)
			{
				linePointArray.push(new Image(Assets.getAtlas().getTexture("dot")));
				map.lineLayer.addChild(linePointArray[i]);
			}
			
			speed = 0.6;
			state = UnitState.SPRAY;
			
			
			super.initializeUnitIcon((map.isNight) ? "LoschpanzerNight" : "Loschpanzer", true, 1);
			
			addNewPointType("targetNormal");
			addNewPointType("HelicopterPickup", true, 20);
			showPointType("targetNormal");
			
//			unitIcon.currentFrame = 0;
			
			//var dropShadow:BlurFilter = BlurFilter.createDropShadow(5);
			
			//this.filter = dropShadow;
//			if (isMyUnit)
//			{
//				tapGesture = new TapGesture(unitIcon);
//				tapGesture.numTouchesRequired = 1;
//			}
			
			unitIconShadow.x = 3;
			unitIconShadow.y = 3;
			
			rotateUnitAlongPath();
		}
		
		override protected function onDoubleTapUnitIcon(event:GestureEvent):void
		{
			trace("Engine.onTapUnitIcon");
			if (tappedOnce)
			{
				if (state == UnitState.NORMAL) 
				{
					state = UnitState.SPRAY;
//					unitIcon.currentFrame = 0;
					sprayWater();
				}
				else
				{
					state = UnitState.NORMAL;
//					unitIcon.currentFrame = 1;
					stopWater();
				}
				sendUpdatedUnit(true);
			}
			tappedOnce = true;
		}
		
		override public function disable(_removeJuggler:Boolean = true):void
		{
			super.disable(_removeJuggler);
			if (_removeJuggler) stopWater();
			
		}
		
		
//		override public function initializeUnit():void
//		{
//			super.initializeUnit();
//			
////			if (isMyUnit)
////			{
////				tapGesture.addEventListener(GestureEvent.GESTURE_RECOGNIZED, onDoubleTapUnitIcon);
////			}
//		}
		
		override public function reset():void
		{
			super.reset();
//			if (isMyUnit) tapGesture.removeEventListener(GestureEvent.GESTURE_RECOGNIZED, onDoubleTapUnitIcon);
			stopWater();
			state = UnitState.SPRAY;
		}
		
		override protected function updatePointTypeByTerrain():void
		{
//			try
//			{			
//				if (targetUnit == null)
//				{
//					var terrainType:Number = map.terrain[Math.round(targetPoint.x/map.blitSize)][Math.round(targetPoint.y/map.blitSize)].type;
//					if (terrainType == TerrainType.WATER_DEEP)
//					{
//						showPointType("targetHelicopterWaterPump");
//					}
//					else
//					{
//						showPointType("targetNormal");
//					}
//			}
//			catch(e:Error)
//			{
//				showPointType("targetNormal");
//			}
		}
		
		override public function advanceTime(passedTime:Number):void
		{
			rotateUnitAlongPath();
			
			if (isDragAndDrop)
			{
				return;
			}
			
			originSprite.x = x;
			originSprite.y = y - 20;
			if (terrainUnderUnit == null && terrainUnderUnitPrevious != null) callout("Look at me, \nI'm super fast!", this, true);
			
			
			if (isCollect) 
			{
				// Add new linePath points
				var point:Point = new Point((touchUnit.globalX - map.x)/map.scaleX,(touchUnit.globalY - map.y)/ map.scaleY);
				targetPoint.x = point.x;
				targetPoint.y = point.y;
				updatePointTypeByTerrain();
				
				map.updateMinimap(targetPoint);
				
				var lastPoint:Point = linePath.points[linePath.points.length-1];
				
				if (lastPoint && Point.distance(point, lastPoint) >= maxDistance)
				{
					calculatePoints(lastPoint, point, linePath);
					updateSpeedAndProgress();
				}
				targetPoint.rotation = linePath.getAngleAtProgress(1);
				sendUpdatedUnit();
			}
			
			updateUnitOnNewTerrain();
			
			if (targetUnit && targetUnit.isOnHelicopter)
			{
				targetUnit.x = -35 * Math.cos(unitIcon.rotation);
				targetUnit.y = -35 * Math.sin(unitIcon.rotation);
			}
			
			if (isCollect) checkIfPickupUnit();
			
			// Follow linePath if it exists
			if (linePath.totalLength > 0)
			{
				// Draw series of dots along unit's linePath
				drawLine();
				
				// Advance unit's progress along linePath
				progress += speedOnPath * TweenMax.globalTimeScale * passedTime*30 * speedVariantBasedOnTerrain;
				if (progress > 1) progress = 1;
				if (linePath.points.length > 1) 
				{
					linePath.renderObjectAt(this, progress, false, 0);
					targetRotation = linePath.angle;
				}
				
				oldProgress = progress;
				
				if (progress == 1) arrivedAtDestination();
			}
			
			//this.scaleX = this.scaleY = Math.min(1, 1/map.scaleX);

		}
		
		protected function checkIfPickupUnit():void
		{
			if (targetUnit && targetUnit.isOnHelicopter) return;
			
			var isUnitAvailableForPickup:Boolean = false;
			var _targetUnit:Unit;
			//targetUnit = null;
			
			if (true)//(bucket.scaleX == 0) // if helicopter has no water, then check for units to pickup
			{
				var cutTeamUnit:CutTeam;
				for (var unitID in map.unitPool[UnitType.CUT_TEAM])
				{
					cutTeamUnit = map.unitPool[UnitType.CUT_TEAM][unitID];
					
					if (cutTeamUnit.parent && Point.distance(new Point(targetPoint.x,targetPoint.y), new Point(cutTeamUnit.x, cutTeamUnit.y)) < 25)
					{
						_targetUnit = cutTeamUnit;
						isUnitAvailableForPickup = true;
						
						
						break;
					}
				}
				
				var hoseTeamUnit:HoseTeam;
				for (var unitID in map.unitPool[UnitType.HOSE_TEAM])
				{
					hoseTeamUnit = map.unitPool[UnitType.HOSE_TEAM][unitID];
					
					if (hoseTeamUnit.parent && Point.distance(new Point(targetPoint.x,targetPoint.y), new Point(hoseTeamUnit.x, hoseTeamUnit.y)) < 25)
					{
						_targetUnit = hoseTeamUnit;
						isUnitAvailableForPickup = true;
						
						
						break;
					}
				}
				
				var civilianUnit:Civilian;
				for (var unitID in map.unitPool[UnitType.CIVILIAN])
				{
					civilianUnit = map.unitPool[UnitType.CIVILIAN][unitID];
					
					if (civilianUnit.parent && Point.distance(new Point(targetPoint.x,targetPoint.y), new Point(civilianUnit.x, civilianUnit.y)) < 25)
					{
						_targetUnit = civilianUnit;
						isUnitAvailableForPickup = true;
						
						
						break;
					}
				}
			}
			
			if (isUnitAvailableForPickup)
			{
				if (targetUnit != _targetUnit) 
				{
					targetUnit = _targetUnit;
					showPointType("HelicopterPickup");
					trace("Pickup");
				}
			}
			else
			{
				if (targetUnit && !targetUnit.isOnHelicopter)
				{
					showPointType("targetNormal");
					targetUnit.targetUnit = null;
					targetUnit = null;
				}
				
			}
			
			//return isUnitAvailableForPickup;
		}
		
		/**
		 *
		 */
		public function stopWater():void
		{
			for (var i:int = 0; i < wetTrees.length; i++)
			{
				if (wetTrees[i])
				{
					wetTrees[i].waterCount--;
					map.blitIt(wetTrees[i], false);
				}
			}
			
			wetTrees = new Array();
		}
		
		/**
		 *
		 */
		public function sprayWater():void
		{
			stopWater();
			
			//trace("spray water");
			var i, j:Number;
			var _terrain:Object;
			var locX:int = Math.floor(this.x / map.blitSize);
			var locY:int = Math.floor(this.y / map.blitSize);
			
//			if (terrainUnderUnit)
//			{
//				if (terrainUnderUnit.type == TerrainType.TREE)
//				{
//					waterRadius = 40;
//				}
//				else
//				{
//					waterRadius = 55;
//				}
//			}
			var spread:Number = Math.ceil(waterRadius / map.blitSize / 2);
			
			for (i = -spread; i < spread; i++)
			{
				for (j = -spread; j < spread; j++)
				{
					try
					{
						_terrain = map.terrain[locX + i][locY + j];
						if (Point.distance(new Point(this.x, this.y), new Point(this.x + map.blitSize * i, this.y + map.blitSize * j)) < waterRadius * map.mapScale / 2)
						{
							wetTrees.push(_terrain);
							_terrain.waterCount++;
							_terrain.wetMultiplier = (1 - (waterIntensity * map.waterResistanceList[_terrain.type]))/_terrain.waterCount;
							//if (_terrain.state != "")
							map.blitIt(_terrain, false);
						}
					}
					catch (err:Error)
					{
					}
				}
			}
		}
		
		
		override protected function updateUnitOnNewTerrain():void
		{
			super.updateUnitOnNewTerrain();
			
			//speed = UnitSpeeds[state][terrainUnderUnit.type][terrainUnderUnit.state]
			if(speedVariantBasedOnTerrain == 0 && terrainUnderUnit.state != TerrainState.FIRE)
			{
				terrainUnderUnitPrevious = null;
			}

			
			if (terrainUnderUnit && terrainUnderUnit != terrainUnderUnitPrevious)
			{
				
				if (state == UnitState.NORMAL)
				{
					if (terrainUnderUnit == null)
					{
						speedVariantBasedOnTerrain = 10;
					}
					else if ((terrainUnderUnit.state == TerrainState.FIRE || terrainUnderUnit.state == TerrainState.FIRE_WET) && terrainUnderUnit.type == TerrainType.TREE)
					{
						if (lastGoodTerrain)
						{
							x = lastGoodTerrain.x;
							y = lastGoodTerrain.y;
						}
						speedVariantBasedOnTerrain = 0;
						//arrivedAtDestination();
						sprayWater();
						if (terrainUnderUnit && terrainUnderUnit != terrainUnderUnitPrevious) 
						{
							//unitIcon.currentFrame = 2;
							terrainUnderUnitPrevious = terrainUnderUnit;
							//							if (redblox.randomizeNumber(0,10) < 3)
							//							{
							var answers:Array = new Array();
							answers.push("We can't go any \nfurther, fire is \ntoo strong!");
							answers.push("It's too hot!");
							answers.push("Are you kidding? \nWe're not going \nin there!");
							answers.push("Let's head another \nway, fire is \nblocking us!");
							answers.push("Whoa! Hot! Hot!");
							callout(answers[redblox.randomizeNumber(0,answers.length-1)], this, true);
							//							}
						}
					}
					else if (isTerrainNearby(TerrainType.ROAD,2))
					{
						speedVariantBasedOnTerrain = 1;
						lastGoodTerrain = new Point(x,y);
						if (linePath.points.length > 1) stopWater();
					}
					else if (isTerrainNearby(TerrainType.GRAVEL,2))
					{
						speedVariantBasedOnTerrain = 1;
						lastGoodTerrain = new Point(x,y);
						if (linePath.points.length > 1) stopWater();
					}
					else if (isTerrainNearby(TerrainType.SAND,2))
					{
						speedVariantBasedOnTerrain = 0.8;
						lastGoodTerrain = new Point(x,y);
						if (linePath.points.length > 1) stopWater();
					}
					else if (isTerrainNearby(TerrainType.GRASS,2))
					{
						speedVariantBasedOnTerrain = 0.9;
						lastGoodTerrain = new Point(x,y);
						if (linePath.points.length > 1) stopWater();
					}
					else if (isTerrainNearby(-1,2, TerrainState.CUT))
					{
						speedVariantBasedOnTerrain = 0.8;
						lastGoodTerrain = new Point(x,y);
						if (linePath.points.length > 1) stopWater();
					}
					else if (isTerrainNearby(-1,2, TerrainState.DEAD))
					{
						speedVariantBasedOnTerrain = 0.8;
						lastGoodTerrain = new Point(x,y);
						if (linePath.points.length > 1) stopWater();
					}
					else if (isTerrainNearby(TerrainType.BUSH,2))
					{
						speedVariantBasedOnTerrain = 0.8;
						lastGoodTerrain = new Point(x,y);
						if (linePath.points.length > 1) stopWater();
					}
					else if (isTerrainNearby(TerrainType.WATER_SHALLOW,2))
					{
						speedVariantBasedOnTerrain = 0.4;
						lastGoodTerrain = new Point(x,y);
						if (linePath.points.length > 1) stopWater();
					}
					else
					{
						//						clearLine();
						//						progress = 0;
						//						linePath.points = new Array();
						//						if (map.lineLayer.contains(targetPoint)) map.lineLayer.removeChild(targetPoint);
						callout("Terrain too thick, \nI can not go further!", this, true);
						arrivedAtDestination();
						if (lastGoodTerrain)
						{
							x = lastGoodTerrain.x;
							y = lastGoodTerrain.y;
						}
						//speedVariantBasedOnTerrain = 0;
					}
				}
				else if (state == UnitState.SPRAY)
				{
//					img.gotoAndStop(1);
					//sprayWater();
//					if (terrainUnderUnit && terrainUnderUnit != terrainUnderUnitPrevious) 
//						unitIcon.currentFrame = 0;
					
					if (terrainUnderUnit == null)
					{
						speedVariantBasedOnTerrain = 10;
					}
					else if ((terrainUnderUnit.state == TerrainState.FIRE || terrainUnderUnit.state == TerrainState.FIRE_WET) && terrainUnderUnit.type == TerrainType.TREE)
					{
						if (lastGoodTerrain)
						{
							x = lastGoodTerrain.x;
							y = lastGoodTerrain.y;
						}
						speedVariantBasedOnTerrain = 0;
						//arrivedAtDestination();
						sprayWater();
						if (terrainUnderUnit && terrainUnderUnit != terrainUnderUnitPrevious) 
						{
							//unitIcon.currentFrame = 2;
							terrainUnderUnitPrevious = terrainUnderUnit;
//							if (redblox.randomizeNumber(0,10) < 3)
//							{
								var answers:Array = new Array();
								answers.push("We can't go any \nfurther, fire is \ntoo strong!");
								answers.push("It's too hot!");
								answers.push("Are you kidding? \nWe're not going \nin there!");
								answers.push("Let's head another \nway, fire is \nblocking us!");
								answers.push("Whoa! Hot! Hot!");
								callout(answers[redblox.randomizeNumber(0,answers.length-1)], this, true);
//							}
						}
					}
					else if (isTerrainNearby(TerrainType.ROAD,2))
					{
						speedVariantBasedOnTerrain = 1;
						lastGoodTerrain = new Point(x,y);
						if (linePath.points.length > 1) stopWater();
					}
					else if (isTerrainNearby(TerrainType.GRAVEL,2))
					{
						speedVariantBasedOnTerrain = 1;
						lastGoodTerrain = new Point(x,y);
						if (linePath.points.length > 1) stopWater();
					}
					else if (isTerrainNearby(TerrainType.SAND,2))
					{
						speedVariantBasedOnTerrain = 0.8;
						lastGoodTerrain = new Point(x,y);
						if (linePath.points.length > 1) stopWater();
					}
					else if (isTerrainNearby(TerrainType.GRASS,2))
					{
						speedVariantBasedOnTerrain = 0.9;
						lastGoodTerrain = new Point(x,y);
						if (linePath.points.length > 1) stopWater();
					}
					else if (isTerrainNearby(-1,2, TerrainState.CUT))
					{
						speedVariantBasedOnTerrain = 0.8;
						lastGoodTerrain = new Point(x,y);
						if (linePath.points.length > 1) stopWater();
					}
					else if (isTerrainNearby(-1,2, TerrainState.DEAD))
					{
						speedVariantBasedOnTerrain = 0.8;
						lastGoodTerrain = new Point(x,y);
						if (linePath.points.length > 1) stopWater();
					}
					else if (isTerrainNearby(TerrainType.BUSH,2))
					{
						speedVariantBasedOnTerrain = 0.8;
						lastGoodTerrain = new Point(x,y);
						if (linePath.points.length > 1) stopWater();
					}
					else if (isTerrainNearby(TerrainType.WATER_SHALLOW,2))
					{
						speedVariantBasedOnTerrain = 0.4;
						lastGoodTerrain = new Point(x,y);
						if (linePath.points.length > 1) stopWater();
					}
					else
					{
//						clearLine();
//						progress = 0;
//						linePath.points = new Array();
//						if (map.lineLayer.contains(targetPoint)) map.lineLayer.removeChild(targetPoint);
						callout("Terrain too thick, \nI can not go further!", this, true);
						arrivedAtDestination();
						if (lastGoodTerrain)
						{
							x = lastGoodTerrain.x;
							y = lastGoodTerrain.y;
						}
						//speedVariantBasedOnTerrain = 0;
					}

				}
			}
		}
		
		/**
		 * 
		 * @param	unit
		 */
		override protected function arrivedAtDestination ():void
		{
			//trace("\nUnit.arrivedAtDestination();");
			clearLine();
			progress = 0;
			linePath.points = new Array();
			map.lineLayer.removeChild(targetPoint);
			sprayWater();
			SoundAS.getSound("BrushTruck").stop();
			if (redblox.isActive) SoundAS.play("HoseTeam");
			
			if (targetUnit && !targetUnit.isOnHelicopter)// && Point.distance(new Point(targetUnit.x, targetUnit.y), new Point(this.x, this.y)) < 5 * mapLayer.mapScale)
			{
				trace("Picking up unit: ");
				targetUnit.x = -35;
				targetUnit.y = 0;
				this.addChild(targetUnit);
				targetUnit.scaleX = targetUnit.scaleY = 0.5;
				targetUnit.isOnHelicopter = true;
				targetUnit.disable();
				targetUnit.unitIcon.currentFrame = 0;
				
				
				showPointType("targetNormal");
			}
			else if (targetUnit && targetUnit.isOnHelicopter)//state == UnitState.WATER_DROP)
			{
				trace("Dropping unit");
				state = UnitState.NORMAL;
				
				map.unitLayer.addChildAt(targetUnit,0);
				targetUnit.x = x;
				targetUnit.y = y;
				
				//if (targetUnit.isMyUnit) 
				targetUnit.scaleX = targetUnit.scaleY = 1;
				//else targetUnit.scaleX = targetUnit.scaleY = 0.6;
				
				targetUnit.targetUnit = null;
				targetUnit.isOnHelicopter = false;
				
				targetUnit.enable();
				targetUnit = null;
				
			}
		} 
		
		
		/**
		 * 
		 * @param	_terrain
		 */
		override public function sendUpdatedUnit(_forceSend:Boolean = false):void 
		{
			if (isMyUnit && (sendCounter >= 5 || _forceSend))
			{
				sendCounter = 0;
				var gameData:Object = new Object();
				gameData.action = GameDataType.UNIT_NEWLINEPATHPOINTS;
				gameData.userID = PlayerIOConnection.myUserID;
				
				gameData.points = linePath.points;
				gameData.speed = speed;
				gameData.speedOnPath = speedOnPath;
				gameData.progress = progress;
				gameData.state = state;
				gameData.speedVariantBasedOnTerrain = speedVariantBasedOnTerrain;
				//				data.isOffScreen = isOffScreen;
				//				data.isEnteringMap = isEnteringMap;
				gameData.id = id;
				gameData.type = type;
				gameData.oldLength = oldLength;
				gameData.oldProgress = oldProgress;
				gameData.x = x;
				gameData.y = y;
				gameData.rotation = unitIcon.rotation;
				gameData.targetPointRotation = targetPoint.rotation;
				gameData.pointName = pointName;
				gameData.playerName = PlayerIOConnection.playerName;
				gameData.isNewLinepath = isNewLinepath;				
				isNewLinepath = false;
				
				map.sendDataToPlayerIO(gameData);
			}
			sendCounter++;
		}
		
		override public function processUnit(_data:Object):void
		{
			//			if (map.isSetupComplete)
			if (true)
			{
				var tempPoints:Array = new Array();
				for (var i:int = 0; i < _data.points.length; i++)
				{
					tempPoints.push(new Point(_data.points[i].x, _data.points[i].y))
				}
				linePath.points = tempPoints;
				speed = _data.speed;
				speedOnPath = _data.speedOnPath;
				progress = _data.progress;
				oldLength = _data.oldLength;
				oldProgress = _data.oldProgress;
				unitIcon.rotation = _data.rotation;
				targetPoint.rotation = _data.targetPointRotation;
				speedVariantBasedOnTerrain = _data.speedVariantBasedOnTerrain;
				state = _data.state;
				
				if (redblox.isActive && _data.isNewLinepath) SoundAS.play("BrushTruck", 0.70 ,redblox.randomizeNumber(500,2000)).fadeTo(0,10000);
					
					
//				if (state == UnitState.NORMAL) 
//				{
//					unitIcon.currentFrame = 1;
//				}
//				else
//				{
//					unitIcon.currentFrame = 0;
//				}

				if (linePath.points.length > 0)
				{
					targetPoint.x = linePath.points[linePath.points.length-1].x;
					targetPoint.y = linePath.points[linePath.points.length-1].y;
					map.lineLayer.addChild(targetPoint);
				}
				else
				{
					if (map.lineLayer.contains(targetPoint)) map.lineLayer.removeChild(targetPoint);
				}
				//this.img.icon.visible = _data.visible;
				udpatePlayerName(_data.playerName);
				userID = _data.userID;
				updateSpeedAndProgress();
				//map.lineLayer.addChild(targetPoint);
				showPointType(_data.pointName);
				updatePointTypeByTerrain();
				clearLine();
				this.x = _data.x;
				this.y = _data.y;
				drawLineCounter = 10;
				drawLine();
				isDragAndDrop = false;
				this.visible = true;
				if (map.unitLayer.contains(this) == false) map.unitLayer.addChildAt(this, 0);
				
				if (targetUnit) // check to see if a helicopter is coming to pick him up
				{
					targetUnit.disable(false);
					targetUnit.targetUnit = null;
					targetUnit = null;
				}
				//if (tempPoints.length > 0) TweenMax.to(endPoint, 0, { x:tempPoints[tempPoints.length - 1].x, y: tempPoints[tempPoints.length - 1].y, autoAlpha:1 } );
			}
		}
		
		override protected function onMouseDownUnit():void
		{
			if (redblox.isActive) SoundAS.play("BrushTruck", 0.70 ,redblox.randomizeNumber(500,2000)).fadeTo(0,10000);
			super.onMouseDownUnit();
			stopWater();
			speedVariantBasedOnTerrain = 1;
		}
		
		override protected function onMouseUpUnit():void
		{
			if (isDragAndDrop)
			{
				if (isTerrainNearby(TerrainType.ROAD,2) || isTerrainNearby(TerrainType.GRAVEL,2))
				{
					callout("I'm a transport/rescue vehicle with water \nspraying capabilities. I can drive through \nGRASS & BRUSH fires unhindered.", this, true, false);
					map.unitLayer.addChildAt(this,0);
					isDragAndDrop = false;
					arrivedAtDestination();
				}
				else
				{
					callout("I'm too far from \na ROAD, place \nme somewhere else!", this, true);
					return;
				}
			}
			
			super.onMouseUpUnit();
		}
		
		override protected function onMouseMoveUnit():void
		{
			tappedOnce = false;
			super.onMouseMoveUnit();
		}
		
		
		override public function dispose():void
		{
			
			super.dispose();
		}
	}
}