package units
{
	import com.greensock.TweenMax;
	import com.greensock.motionPaths.LinePath2D;
	
	import flash.desktop.Icon;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.geom.Point;
	import flash.utils.ByteArray;
	
	import connections.PlayerIOConnection;
	
	import feathers.controls.Label;
	import feathers.controls.Radio;
	import feathers.core.ToggleGroup;
	import feathers.layout.VerticalLayout;
	
	import maps.GameDataType;
	import maps.Map;
	import maps.TerrainState;
	import maps.TerrainType;
	
	import starling.animation.IAnimatable;
	import starling.core.Starling;
	import starling.display.Image;
	import starling.display.MovieClip;
	import starling.display.Sprite;
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.filters.BlurFilter;
	import starling.textures.Texture;
	import starling.utils.Line;
	
	import treefortress.sound.SoundAS;
	
	public class RetardantBomber extends Unit
	{
		protected var isPumpingWater:Boolean;
		protected var bucket:Image;
		protected var bucketWidth:Number = 35;
		protected var speedWaterPump:Number = 1/30/70;		// 1 iteration / 30 fps / X sec = incremental delta of pumping water for X secs
		protected var speedWaterDrop:Number = 1/30/6;		// 1 iteration / 30 fps / X sec = decremental delta of dropping water over X secs;
		protected var waterDropCounter:int = 0; 			// incremental counter used with "waterDropFrequency"
		protected var waterDropFrequency:int = 10;		// to drop water 1 per second, make it 30 since there's 30fps
		protected var pumpWaterPointArray:Array;
		protected var dropWaterPointArray:Array;
		protected var shadowDistance:Number = 1;
		
		private var prevState:int = 0;
		
		private var radioGroupMenu:ToggleGroup = new ToggleGroup();
		private var radioActivated:Radio = new Radio();
		private var radioDeactivated:Radio = new Radio();
		private var labelTimer:Label;
		
		public function RetardantBomber( _map:Map, _isMyUnit:Boolean, _id:int)
		{
			type = UnitType.RETARDANT_BOMBER;
//			TweenPlugin.activate([DropShadowFilterPlugin]);
			
			super(_map, _isMyUnit, _id);
		}
		
		override public function toggleNight():void
		{
			var currentScale:Number = 1;
			if (bucket) 
			{
				currentScale = bucket.scaleX;
				bucket.removeFromParent(true);
			}
			bucket = new Image(Assets.getAtlas().getTexture((map.isNight) ? "BucketRetardantBomberNight" : "BucketRetardantBomber"));
			bucket.alignPivot();
			bucket.scaleX = bucket.scaleY = currentScale;
			
			super.initializeUnitIcon((map.isNight) ? "RetardantBomberNight" : "RetardantBomber", true, 24);
			
			iconUnitContainer.addChild(unitIconShadow);			
			iconUnitContainer.addChild(bucket);
			addChild(unitIconCollision);
			iconUnitContainer.addChild(unitIcon);
		}
		
		
		override protected function initializeOnce():void
		{
			super.initializeOnce();
			
			if (bucket) bucket.removeFromParent(true);
			bucket = new Image(Assets.getAtlas().getTexture((map.isNight) ? "BucketRetardantBomberNight" : "BucketRetardantBomber"));
			bucket.alignPivot();
			
			speed = 1.6;
			
			super.initializeUnitIcon((map.isNight) ? "RetardantBomberNight" : "RetardantBomber", true, 24);
			
			addNewPointType("RetardantBomberDropZone", true, 5);
			pointType["RetardantBomberDropZone"].x = -25;
			
			radioGroupMenu.addEventListener( Event.CHANGE, changeUnitState );
			radioActivated.label = "ACTIVATED";
			radioActivated.toggleGroup = radioGroupMenu;
			
			radioDeactivated.label = "DEACTIVATED";
			radioDeactivated.toggleGroup = radioGroupMenu;
			
			
			var menuLayout:VerticalLayout = new VerticalLayout();
			menuLayout.gap = 20;
			
			containerMenu.layout = menuLayout;
			
			containerMenu.addChild(radioDeactivated);
			containerMenu.addChild(radioActivated);
			
			//var dropShadow:BlurFilter = BlurFilter.createDropShadow(15);
			
			//this.filter = dropShadow;
			
//			TweenMax.to(this,0,{dropShadowFilter:{blurX:5, blurY:5, distance:5, alpha:0.6}});
			
			unitIconShadow.x = 20 * shadowDistance;
			unitIconShadow.y = 20 * shadowDistance;
			
		}
		
		override public function initializeUnit():void
		{
			super.initializeUnit();
			
			callout("Place me anywhere", this, true, false);
			iconUnitContainer.addChild(unitIconShadow);			
			iconUnitContainer.addChild(bucket);
			addChild(unitIconCollision);
			iconUnitContainer.addChild(unitIcon);
			
			bucket.scaleX = bucket.scaleY = 0;
//			pumpWaterPointArray = new Array();
			
			dropWaterPointArray = new Array();
			for (var i:int = 0; i < 6; i++)
			{
				dropWaterPointArray.push(new MovieClip(Assets.getAtlas().getTextures("RetardantDropCircle_"),1));
				TweenMax.delayedCall(1-0.1*i,starling.core.Starling.juggler.add,[dropWaterPointArray[i]]);
				
				map.lineLayer.addChild(dropWaterPointArray[i]);
				dropWaterPointArray[i].visible = false;
			}		
		}
		
		public function changeUnitState(e:Event):void
		{
			trace("CutTeam.onTapUnitIcon");
			if (!isDragAndDrop)
			{
				prevState = state;
				if (radioGroupMenu.selectedItem == radioDeactivated)
				{
					targetPoint.alpha = 0.4;
				}
				else if (radioGroupMenu.selectedItem == radioActivated)
				{
					targetPoint.alpha = 1;
				}
				sendUpdatedUnit(true);
			}
		}
		

		
		override protected function updatePointTypeByTerrain():void
		{
//			try
//			{
//				var terrainType:Number = map.terrain[Math.round(targetPoint.x/map.blitSize)][Math.round(targetPoint.y/map.blitSize)].type;
//				if (bucket.scaleX > 0)//map.checkToSeeIfThereIsFire(targetPoint, bucketWidth*bucket.scaleX))
//				{
//					showPointType("RetardantBomberDropZone");
//				}
//				else
//				{
//					showPointType("targetNormal");
//				}
//			}
//			catch(e:Error)
//			{
//				showPointType("targetNormal");
//			}
		}
		

		
		
		/**
		 * 
		 * @param	unit
		 */
		override protected function arrivedAtDestination ():void
		{
			super.arrivedAtDestination();
			
			clearLine();
			
			trace("Bomber.arrivedAtDestination();");
			trace("> state: " + state);
//			if (bucket.scaleX > 0 && targetPoint.alpha == 1)
//			{
//				trace("> there is fire here");
//				state = UnitState.WATER_DROP;
//				
//				if (redblox.isActive) SoundAS.play("WaterBomberDropWater");
//
//			}
//			else if (map.terrainTypeUnderPoint(this, TerrainType.WATER_DEEP))
//			{
//				trace("> there is water here");
//				state = UnitState.WATER_PUMP;
//			}
			
		} 
		
		override public function advanceTime(passedTime:Number):void
		{
			//trace("Bomber.advanceTime(): " + isCollect);
			super.advanceTime(passedTime);
			super.rotateUnitAlongPath();
			
			if (isDragAndDrop || state == UnitState.DEAD)
			{
				return;
			}
			
			if (terrainUnderUnit == null)
			{
				state = UnitState.OFFSCREEN;
//				if (terrainUnderUnitPrevious != null) 
//					callout("Off-Screen PUMP ZONE takes 70 seconds to fill up.", this, true);
			}
			else if (terrainUnderUnit && state == UnitState.OFFSCREEN)
			{
				state = UnitState.NORMAL;
			}
			
			if (progress + speedOnPath >= 1 || linePath.points.length <= 1 || linePath.totalLength < 5) {  
//				lines.graphics.clear();
				//if (!isOffScreen && panel.panelState.visible == false)
//				if (!isOffScreen && !isWaitingToLaunch)
				if (this.x > -50 && this.x < map.terrainWidth+50 && y > -50 && y < map.terrainHeight+50)
				{
					//trace("unitIcon.rotation: " + unitIcon.rotation);
					//trace("targetPoint.rotation: " + targetPoint.rotation);
					if (!isNaN(targetPoint.rotation)) 
					{
						targetRotation = targetPoint.rotation * 180/Math.PI;
					}
					
					rotateUnitAlongPath();
					
					var angle:Number = unitIcon.rotation; // * Math.PI / 180;
					
					//var lastPoint:Point = points[points.length-1]; 
					//if (redblox.fps < 20)
					//{
					//this.x += speed * Math.cos(angle) * timeScale * TweenMax.globalTimeScale; 
					//this.y += speed * Math.sin(angle) * timeScale * TweenMax.globalTimeScale;
					//}
					//else
					//{
					this.x += speed * (passedTime * 30) * Math.cos(angle) * TweenMax.globalTimeScale;
					this.y += speed * (passedTime * 30) * Math.sin(angle) * TweenMax.globalTimeScale;
					//}
					//endPoint.visible = false;
				}
			} 
			
			
//			if (state == UnitState.WATER_PUMP)
//			{
//				waterPump(passedTime);
//				if (shadowDistance > 0.05){
//					shadowDistance = Math.max(0.05, shadowDistance - 0.01);
//					unitIconShadow.x = unitIconShadow.y = 20 * shadowDistance;
//				}
//			}
			if (state == UnitState.OFFSCREEN)
			{
				if (!this.contains(countdownPopup)) addChild(countdownPopup);
				
				if (bucket.scaleX < 1)
				{
					waterPump(passedTime);
				}
				if (bucket.scaleX == 1)
				{
					shadowDistance = Math.max(0.05, shadowDistance + 0.01);
					unitIconShadow.x = unitIconShadow.y = 20 * shadowDistance;
				}
				if (shadowDistance > 0.05){
					shadowDistance = Math.max(0.05, shadowDistance - 0.01);
					unitIconShadow.x = unitIconShadow.y = 20 * shadowDistance;
				}
				
			}
			else if (state == UnitState.WATER_DROP)
			{
				waterDrop(passedTime);
			}
			else if (state == UnitState.NORMAL)
			{
				if (this.contains(countdownPopup)) removeChild(countdownPopup);
				
				if (shadowDistance < 1){
					shadowDistance = Math.min(1, shadowDistance + 0.01);
					unitIconShadow.x = unitIconShadow.y = 20 * shadowDistance;
				}
			}
		}
		
//		protected function pickupUnit(targetUnit:Sprite):void
//		{
//			// TODO Auto Generated method stub
//			
//		}
//		
		protected function waterPump(passedTime:Number):void
		{
			//trace("waterPump()");
			if (state == UnitState.OFFSCREEN && bucket.scaleX < 1)
			{
				bucket.scaleX = bucket.scaleY = bucket.scaleX + (speedWaterPump) * passedTime * 30;
				udpateCountdown((70 - Math.round(bucket.scaleX * 70)) + "s to FULL");
			}
			else if (state == UnitState.OFFSCREEN)
			{
			}
			else
			{
				state = UnitState.NORMAL;
				
			}
			
			if (bucket.scaleX >= 1) 
			{
				udpateCountdown("DONE!");
				trace("Retardant Bomber ready!");
				callout("Retardant\nBomber\nFULL!",this,true,false);
				bucket.scaleX = 1;
			}
		}
		
		protected function waterDrop(passedTime:Number):void
		{
			if (bucket.scaleX > 0)
			{
				if (waterDropCounter % waterDropFrequency == 0) map.dropRetardant(this, Math.min(bucketWidth,Math.round(bucketWidth*bucket.scaleX*2)), 1.5);
				bucket.scaleX = bucket.scaleY = bucket.scaleX - speedWaterDrop * passedTime*30 * TweenMax.globalTimeScale;
				waterDropCounter++;
			}
			else
			{
				waterDropCounter = 0;
				state = prevState;
			}
		}		
		
		override public function sendUpdatedUnit(_forceSend:Boolean = false):void 
		{
			if (isMyUnit && (sendCounter >= 5 || _forceSend))
			{
				sendCounter = 0;
				var gameData:Object = new Object();
				gameData.action = GameDataType.UNIT_NEWLINEPATHPOINTS;
				gameData.userID = PlayerIOConnection.myUserID;
				
				gameData.points = linePath.points;
				gameData.speed = speed;
				gameData.speedOnPath = speedOnPath;
				gameData.progress = progress;
				//				data.isOffScreen = isOffScreen;
				//				data.isEnteringMap = isEnteringMap;
				gameData.id = id;
				gameData.type = type;
				gameData.oldLength = oldLength;
				gameData.oldProgress = oldProgress;
				gameData.x = x;
				gameData.y = y;
				gameData.rotation = unitIcon.rotation;
				gameData.targetPointRotation = targetPoint.rotation;
				gameData.pointName = pointName;
				gameData.playerName = PlayerIOConnection.playerName;
				gameData.bucketScaleX = bucket.scaleX;
				gameData.alphaTargetPoint = targetPoint.alpha;
				gameData.state = state;
				gameData.isNewLinepath = isNewLinepath;				
				isNewLinepath = false;

				map.sendDataToPlayerIO(gameData);
			}
			sendCounter++;
		}
		
		override public function processUnit(_data:Object):void
		{
			//			if (map.isSetupComplete)
			if (true)
			{
				var tempPoints:Array = new Array();
				for (var i:int = 0; i < _data.points.length; i++)
				{
					tempPoints.push(new Point(_data.points[i].x, _data.points[i].y))
				}
				linePath.points = tempPoints;
				speed = _data.speed;
				speedOnPath = _data.speedOnPath;
				progress = _data.progress;
				oldLength = _data.oldLength;
				oldProgress = _data.oldProgress;
				bucket.scaleX = bucket.scaleY = _data.bucketScaleX;
				targetPoint.alpha = _data.alphaTargetPoint;
				state = _data.state;
				
				
				if (redblox.isActive && _data.isNewLinepath) SoundAS.play("WaterBomberDropWater").fadeTo(0,7000);
				
				//				isEnteringMap = _data.isEnteringMap;
				//				isOffScreen = _data.isOffScreen;
				//linePath.points = tempPoints;
				//this.x = _data.x;
				//this.y = _data.y;
				unitIcon.rotation = _data.rotation;
				targetPoint.rotation = _data.targetPointRotation;
				if (linePath.points.length > 0)
				{
					targetPoint.x = linePath.points[linePath.points.length-1].x;
					targetPoint.y = linePath.points[linePath.points.length-1].y;
					map.lineLayer.addChild(targetPoint);
				}
				else
				{
					if (map.lineLayer.contains(targetPoint)) map.lineLayer.removeChild(targetPoint);
				}
				//this.img.icon.visible = _data.visible;
				udpatePlayerName(_data.playerName);
				userID = _data.userID;
				updateSpeedAndProgress();
				//map.lineLayer.addChild(targetPoint);
				showPointType(_data.pointName);
				updatePointTypeByTerrain();
				clearLine();
				this.x = _data.x;
				this.y = _data.y;
				drawLineCounter = 10;
				drawLine();
				isDragAndDrop = false;
				this.visible = true;
				if (map.unitLayer.contains(this) == false) map.unitLayer.addChild(this);
				
				//if (tempPoints.length > 0) TweenMax.to(endPoint, 0, { x:tempPoints[tempPoints.length - 1].x, y: tempPoints[tempPoints.length - 1].y, autoAlpha:1 } );
			}
			else
			{
				//				this.x = _data.x;
				//				this.y = _data.y;
				//				this.img.rotation = _data.rotation;
				//				this.img.icon.visible = _data.visible;
			}
		}
		
		override protected function onMouseMoveUnit():void
		{
			super.onMouseMoveUnit();
		}
		
		override protected function onMouseUpUnit():void
		{
			if (isDragAndDrop) 
			{
				callout("Retardant bombers can \nonly fill up OFF SCREEN. \nFire retardant is best dropped \nin front of a fire as a \nliquid fire-line.", this, true, false);
				map.unitLayer.addChild(this);
			}
			super.onMouseUpUnit();
			if (bucket.scaleX > 0) linePath.removePointByIndex(linePath.points.length-1);
			
			map.panelUnitMenu.show(containerMenu, this, "Retardant Bomber");
		}
		
		private function activatePointDirection():void
		{
			map.addEventListener(TouchEvent.TOUCH, onTouchMap);
		}
		
		private function onTouchMap(e:TouchEvent):void
		{
			var touch:Touch = e.getTouch(map);
			if (touch && touch.phase == TouchPhase.BEGAN)
			{
//				onMouseMoveTargetPoint();
				var point:Point = new Point((touch.globalX - map.x)/map.scaleX,(touch.globalY - map.y)/ map.scaleY);
				
				targetPoint.rotation = redblox.point_direction(targetPoint.x, targetPoint.y, point.x, point.y) * Math.PI/180;
				map.removeEventListener(TouchEvent.TOUCH, onTouchMap);

//				var point:Point = this.globalToLocal(new Point(touch.globalX, touch.globalY))
//				
//				targetPoint.rotation = redblox.point_direction(touch.globalX, touch.globalY, this.x, this.y);
			}
//			else if (touch.phase == TouchPhase.ENDED)
//			{
//			}
//			else if (touch.phase == TouchPhase.MOVED)
//			{
//				onMouseMoveTargetPoint();
//			}
		}
		
		override protected function onMouseDownUnit():void
		{
			if (redblox.isActive) SoundAS.play("WaterBomberDropWater").fadeTo(0,7000);
			state = UnitState.NORMAL;
			super.onMouseDownUnit();
		}
		override protected function onMouseDownTargetPoint():void
		{
//			if (!isDragAndDrop)
//			{
				//trace("\nBomber.onMouseMoveTargetPoint()");
				//isCollect = true;
				if (!targetPoint.contains(pointType["RetardantBomberDropZone"]) || targetPoint.alpha != 1)
				{
					isCollect = true;
					map.openMinimap();
				}
//				if (targetPoint.contains(pointType["RetardantBomberDropZone"]))
//				{
//					//activatePointDirection();
//				}
				trace(" > isCollect: " + isCollect);
//			}
		}
		override protected function onMouseMoveTargetPoint():void
		{
//			if (!isDragAndDrop)
//			{
				trace("\nBomber.onMouseMoveTargetPoint()");
				
				if (!isCollect)
				{
					var point:Point = new Point((touchUnit.globalX - map.x)/map.scaleX,(touchUnit.globalY - map.y)/ map.scaleY);
					
					targetPoint.rotation = redblox.point_direction(targetPoint.x, targetPoint.y, point.x, point.y) * Math.PI/180;
				}
				trace(" > isCollect: " + isCollect);
//			}
		}
		
		override protected function onMouseUpTargetPoint():void
		{
			super.onMouseUpTargetPoint();
			
			if (bucket.scaleX > 0) linePath.removePointByIndex(linePath.points.length-1);
//			if (targetPoint.contains(pointType["BomberPumpZone"]))
//			{
//				redblox.savedGameInfo.data.calloutBomberPumpZone = true;
//				redblox.savedGameInfo.flush();
//			}
			if (targetPoint.contains(pointType["RetardantBomberDropZone"]))
			{
				redblox.savedGameInfo.data.calloutBomberDropZone = true;
				redblox.savedGameInfo.flush();
			}
			
			map.panelUnitMenu.show(containerMenu, this, "Retardant Bomber");
		}
		
		override protected function clearLine():void
		{
			super.clearLine();
			
			if (dropWaterPointArray)
			{
				for (var i:int = 0; i < dropWaterPointArray.length; i++)
				{
					dropWaterPointArray[i].visible = false;
				}
			}			
		}
		
		override protected function drawLine():void
		{
			if (linePath.points.length > 0)
			{
				drawLineCounter = 0;
				var points:Number = linePath.totalLength / gap;
				var num:Number = 1 / points;
				var angle:Number;
				
				//spread the points evenly  
				var p:Number = 0;
				var point:Point;
				var prevPoint:Point;
				
				var currentDot:int = 0;
				
				var upToPoints:Array = linePath.getAllPointsUpToProgress(progress);
				var afterPoints:Array = linePath.getAllPointsAfterProgress(progress);
				
				for (var i:int = 0; i < dropWaterPointArray.length; i++)
				{
					dropWaterPointArray[i].visible = false;
				}
				
				for (var i:int = 0; i < upToPoints.length; i++)
				{
					linePointArray[currentDot].visible = false;
					currentDot++;
				}
				
				var lastPoints:int;
				for (var i:int = 0; i < afterPoints.length; i++)
				{
					if (linePointArray[currentDot])
					{
						if (!linePointArray[currentDot].visible) 
						{
							linePointArray[currentDot].x = afterPoints[i].x - 8;
							linePointArray[currentDot].y = afterPoints[i].y - 8;
							linePointArray[currentDot].visible = true;
							linePointArray[currentDot].alpha = 1;
							if (!isMyUnit) TweenMax.to(linePointArray[currentDot], 1, {alpha:0.4, delay:0.02 * currentDot, overwrite:true});
							
						}
						currentDot++;
					}
					else
					{
						break;
					}
					
				}
				if (bucket.scaleX > 0 && (radioGroupMenu.selectedItem == radioActivated))
				{
					currentDot = 0;
					var everyOtherDot:int = 0
					for (var i:int = afterPoints.length; i > afterPoints.length-Math.ceil(14*bucket.scaleX); i--)
					{
						//					lastPoints = linePointArray.length - currentDot;
						//					trace("lastPoints: " + currentDot);
						//					trace("dropWaterPointArray[currentDot]: " + dropWaterPointArray[currentDot]);
						//					trace("afterPoints[i]: " + afterPoints[i]);
						if ((everyOtherDot % 2 == 0) && afterPoints[i])
						{
							dropWaterPointArray[currentDot].x = afterPoints[i].x - 20;
							dropWaterPointArray[currentDot].y = afterPoints[i].y - 20;
							dropWaterPointArray[currentDot].visible = true;
							currentDot++;
						}
						everyOtherDot++;
					}
				}				
				if ((radioGroupMenu.selectedItem == radioActivated) && afterPoints.length < Math.ceil(12*bucket.scaleX) && !isCollect && bucket.scaleX > 0)
				{
					if (state != UnitState.WATER_DROP && redblox.isActive) SoundAS.play("WaterBomberDropWater").fadeTo(0,7000);
					state = UnitState.WATER_DROP;
				}
			}
//			drawLineCounter++;
		}		
	}
}