package units
{
	import com.greensock.TweenMax;
	import com.greensock.motionPaths.LinePath2D;
	
	import flash.geom.Point;
	import flash.utils.Dictionary;
	
	import connections.PlayerIOConnection;
	
	import feathers.controls.Button;
	import feathers.controls.Callout;
	import feathers.controls.Radio;
	import feathers.controls.ScrollContainer;
	import feathers.core.ToggleGroup;
	import feathers.layout.VerticalLayout;
	
	import maps.GameDataType;
	import maps.Map;
	import maps.TerrainState;
	import maps.TerrainType;
	
	import org.gestouch.events.GestureEvent;
	import org.gestouch.gestures.TapGesture;
	
	import starling.animation.IAnimatable;
	import starling.core.Starling;
	import starling.display.Image;
	import starling.display.MovieClip;
	import starling.display.Sprite;
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.filters.BlurFilter;
	import starling.filters.ColorMatrixFilter;
	
	import treefortress.sound.SoundAS;
	import treefortress.sound.SoundInstance;
	import treefortress.sound.SoundManager;

	
	public class Smokejumper extends Unit
	{
		private var tappedOnce:Boolean = false;
		
		private var jumpButton:Button = new Button();
		private var radioGroupMenu:ToggleGroup = new ToggleGroup();
		private var radioCut:Radio = new Radio();
		private var radioRun:Radio = new Radio();
		
		
		public function Smokejumper(_map:Map, _isMyUnit:Boolean, _id:int)
		{
			type = UnitType.SMOKEJUMPER;
			super(_map, _isMyUnit, _id);
		}
		
		override public function toggleNight():void
		{
			super.initializeUnitIcon((map.isNight) ? "SmokejumperNight" : "Smokejumper", false);
		}
		
		override public function initializeUnit():void
		{
			super.initializeUnit();
			
			callout("Place me anywhere", this, true, false);
		}
		
		override protected function initializeOnce():void
		{
			// Add dots to use for line path
			linePointArray = new Array();
			for (var i:int = 0; i < numDots; i++)
			{
				linePointArray.push(new Image(Assets.getAtlas().getTexture("dot")));
				map.lineLayer.addChild(linePointArray[i]);
			}
			
			speed = 0.55;
			state = UnitState.CUT;
			
			super.initializeUnitIcon((map.isNight) ? "SmokejumperNight" : "Smokejumper", false);
			addNewPointType("targetNormal");
			showPointType("targetNormal");
			
			radioGroupMenu.addEventListener( Event.CHANGE, changeUnitState );
			radioCut.label = "Cut";
			radioCut.toggleGroup = radioGroupMenu;
			
			radioRun.label = "Run";
			radioRun.toggleGroup = radioGroupMenu;
			
			jumpButton.label = "Jump";
			jumpButton.isEnabled = false;
			jumpButton.addEventListener(starling.events.Event.TRIGGERED, onButtonClicked);
			
			var menuLayout:VerticalLayout = new VerticalLayout();
			menuLayout.gap = 20;
			
			containerMenu.layout = menuLayout;
			
			containerMenu.addChild(jumpButton);
			containerMenu.addChild(radioRun);
			containerMenu.addChild(radioCut);
			
			radioRun.scaleX = radioRun.scaleY = radioCut.scaleX = radioCut.scaleY = 1.5;
			
			unitIconShadow.x = 3;
			unitIconShadow.y = 3;
			
			rotateUnitAlongPath(false);
		}
		
		public function changeUnitState(e:Event):void
		{
			trace("CutTeam.onTapUnitIcon");
			if (!isDragAndDrop)
			{
				if (radioGroupMenu.selectedItem == radioRun)
				{
					state = UnitState.NORMAL;
					unitIcon.currentFrame = 1;
				}
				else if (radioGroupMenu.selectedItem == radioCut)
				{
					state = UnitState.CUT;
					unitIcon.currentFrame = 0;
				}
				sendUpdatedUnit(true);
			}
		}
		
		private function onButtonClicked(e:starling.events.Event):void
		{
			var buttonClicked:Button = e.target as Button;
			
			if (buttonClicked == jumpButton)
			{
				// Jumping out of aircraft
			}
		}
		
		
//		override public function initializeUnit():void
//		{
//			super.initializeUnit();
////			if (isMyUnit)
////			{
////				tapGesture.addEventListener(GestureEvent.GESTURE_RECOGNIZED, onDoubleTapUnitIcon);
////			}
//		}
		
		override public function reset():void
		{
			super.reset();
			state = UnitState.CUT;
//			if (isMyUnit) tapGesture.removeEventListener(GestureEvent.GESTURE_RECOGNIZED, onDoubleTapUnitIcon);

		}
		
		override protected function updatePointTypeByTerrain():void
		{
			try
			{				
				var terrainType:Number = map.terrain[Math.round(targetPoint.x/map.blitSize)][Math.round(targetPoint.y/map.blitSize)].type;
				if (terrainType == TerrainType.WATER_DEEP)
				{
					showPointType("targetHelicopterWaterPump");
				}
				else
				{
					showPointType("targetNormal");
				}
			}
			catch(e:Error)
			{
				showPointType("targetNormal");
			}
		}
		
		override public function advanceTime(passedTime:Number):void
		{
			rotateUnitAlongPath(false);
			
			if (isDragAndDrop)
			{
				return;
			}
			
			if (terrainUnderUnit == null && terrainUnderUnitPrevious != null) callout("Look at me, \nI'm super fast!", this, true);

			
			if (isCollect) 
			{
				// Add new linePath points
				var point:Point = new Point((touchUnit.globalX - map.x)/map.scaleX,(touchUnit.globalY - map.y)/ map.scaleY);
				targetPoint.x = point.x;
				targetPoint.y = point.y;
				updatePointTypeByTerrain();
				
				map.updateMinimap(targetPoint);
				
				var lastPoint:Point = linePath.points[linePath.points.length-1];
				
				if (lastPoint && Point.distance(point, lastPoint) >= maxDistance)
				{
					calculatePoints(lastPoint, point, linePath);
					updateSpeedAndProgress();
				}
				targetPoint.rotation = linePath.getAngleAtProgress(1);
				sendUpdatedUnit();
			}
			
			updateUnitOnNewTerrain();
			
			// Follow linePath if it exists
			if (linePath.totalLength > 0)
			{
				// Draw series of dots along unit's linePath
				drawLine();
				
				// Advance unit's progress along linePath
				progress += speedOnPath * TweenMax.globalTimeScale * passedTime*30 * speedVariantBasedOnTerrain;
				if (progress > 1) progress = 1;
				if (linePath.points.length > 1) 
				{
					linePath.renderObjectAt(this, progress, false, 0);
					targetRotation = linePath.angle;
				}
				
				oldProgress = progress;
				
				if (progress == 1) arrivedAtDestination();
			}
			
			//this.scaleX = this.scaleY = Math.min(1, 1/map.scaleX);

		}
		
		override protected function updateUnitOnNewTerrain():void
		{
			super.updateUnitOnNewTerrain();
			
			//speed = UnitSpeeds[state][terrainUnderUnit.type][terrainUnderUnit.state]
			if(speedVariantBasedOnTerrain == 0 && terrainUnderUnit.state != TerrainState.FIRE)
			{
				terrainUnderUnitPrevious = null;
			}

			if (terrainUnderUnit && terrainUnderUnit != terrainUnderUnitPrevious)
			{
				if (state == UnitState.NORMAL)
				{
					if (terrainUnderUnit == null)
					{
						speedVariantBasedOnTerrain = 10;
					}
					else if (terrainUnderUnit.state == TerrainState.FIRE || terrainUnderUnit.state == TerrainState.FIRE_WET)
					{
						speedVariantBasedOnTerrain = 0;
//						arrivedAtDestination();
						if (terrainUnderUnit && terrainUnderUnit != terrainUnderUnitPrevious) 
						{
							unitIcon.currentFrame = 2;
							terrainUnderUnitPrevious = terrainUnderUnit;
							if (redblox.randomizeNumber(0,10) < 3)
							{
								var answers:Array = new Array();
								answers.push("We can't go \nany further, fire \nis too strong!");
								answers.push("It's too hot!");
								answers.push("Are you kidding? \nWe're not going \nin there!");
								answers.push("Let's head another \nway, fire is \nblocking us!");
								answers.push("Whoa! Hot! Hot!");
								callout(answers[redblox.randomizeNumber(0,answers.length-1)], this, true);
							}
						}
					}
					else if (terrainUnderUnit.state == TerrainState.DEAD || terrainUnderUnit.state == TerrainState.DEAD_WET)
						speedVariantBasedOnTerrain = 0.5;
					else if (terrainUnderUnit.state == TerrainState.BURNT || terrainUnderUnit.state == TerrainState.BURNT_WET)
						speedVariantBasedOnTerrain = 0.8;
					else if (terrainUnderUnit.state == TerrainState.CUT || terrainUnderUnit.state == TerrainState.CUT_WET)
						speedVariantBasedOnTerrain = 1;
					else if (terrainUnderUnit.type == TerrainType.WATER_DEEP)
						speedVariantBasedOnTerrain = 0.05;
					else if (terrainUnderUnit.type == TerrainType.WATER_SHALLOW)
						speedVariantBasedOnTerrain = 0.75;
					else if (isTerrainNearby(TerrainType.ROAD,1))
						speedVariantBasedOnTerrain = 1.2;
					else if (isTerrainNearby(TerrainType.GRAVEL,1))
						speedVariantBasedOnTerrain = 1.2;
					else if (isTerrainNearby(TerrainType.SAND,1))
						speedVariantBasedOnTerrain = 1;
					else if (terrainUnderUnit.type == TerrainType.GRASS)
						speedVariantBasedOnTerrain = 1;
					else if (terrainUnderUnit.type == TerrainType.BUSH)
						speedVariantBasedOnTerrain = 0.75; //0.80;
					else if (terrainUnderUnit.type == TerrainType.TREE)
						speedVariantBasedOnTerrain = 0.5; // 0.5;
					else if (terrainUnderUnit.type == TerrainType.MARSH)
						speedVariantBasedOnTerrain = 0.3;
					else
						speedVariantBasedOnTerrain = 1;
				}
				else if (state == UnitState.CUT)
				{
//					img.gotoAndStop(1);
					if (terrainUnderUnit && terrainUnderUnit != terrainUnderUnitPrevious) 
						unitIcon.currentFrame = 0;
					
					if (terrainUnderUnit == null)
					{
						speedVariantBasedOnTerrain = 10;
					}
					else if (terrainUnderUnit.state == TerrainState.FIRE || terrainUnderUnit.state == TerrainState.FIRE_WET)
					{
						speedVariantBasedOnTerrain = 0;
//						arrivedAtDestination();
						if (terrainUnderUnit && terrainUnderUnit != terrainUnderUnitPrevious) 
						{
							unitIcon.currentFrame = 2;
							terrainUnderUnitPrevious = terrainUnderUnit;
//							if (redblox.randomizeNumber(0,10) < 3)
//							{
								var answers:Array = new Array();
								answers.push("We can't go \nany further, fire \nis too strong!");
								answers.push("It's too hot!");
								answers.push("Are you kidding? \nWe're not going \nin there!");
								answers.push("Let's head another \nway, fire is \nblocking us!");
								answers.push("Whoa! Hot! Hot!");
								callout(answers[redblox.randomizeNumber(0,answers.length-1)], this, true);
//							}
						}
					}
					else if (terrainUnderUnit.state == TerrainState.DEAD || terrainUnderUnit.state == TerrainState.DEAD_WET)
						speedVariantBasedOnTerrain = 0.5;
					else if (terrainUnderUnit.state == TerrainState.BURNT || terrainUnderUnit.state == TerrainState.BURNT_WET)
						speedVariantBasedOnTerrain = 0.8;
					else if (terrainUnderUnit.type == TerrainType.WATER_DEEP)
						speedVariantBasedOnTerrain = 0.05;
					else if (terrainUnderUnit.type == TerrainType.WATER_SHALLOW)
						speedVariantBasedOnTerrain = 0.75;
					else if (isTerrainNearby(TerrainType.ROAD,1))
						speedVariantBasedOnTerrain = 0.5;
					else if (isTerrainNearby(TerrainType.GRAVEL,1))
						speedVariantBasedOnTerrain = 0.5;
					else if (isTerrainNearby(TerrainType.SAND,1))
						speedVariantBasedOnTerrain = 0.5;
					else if (terrainUnderUnit.type == TerrainType.GRASS)
						speedVariantBasedOnTerrain = 0.5;
					else if (terrainUnderUnit.type == TerrainType.BUSH)
						speedVariantBasedOnTerrain = 0.3; //0.4;
					else if (terrainUnderUnit.type == TerrainType.TREE)
						speedVariantBasedOnTerrain = 0.10; // 0.2;
					else if (terrainUnderUnit.type == TerrainType.MARSH)
						speedVariantBasedOnTerrain = 0.3;
						//else if (currentTerrain.state == "Cut")
						//{
						//
						//}
					else
						speedVariantBasedOnTerrain = 1;
					
					if ((terrainUnderUnitPrevious == undefined || terrainUnderUnitPrevious.type != TerrainType.TREE) && (terrainUnderUnit && terrainUnderUnit.type == TerrainType.TREE))
					{
						var answers:Array = new Array();
						answers.push("These dense trees\n are slow to \ncut through!");
						answers.push("Is there an \neasier path?");
						answers.push("A bulldozer would \nbe nice right \nabout now!");
						answers.push("I'm a lot faster \ncutting grasslands \nor bushes!");
						answers.push("I hope I don't \nhave to cut much more \nof this dense forest.");
						callout(answers[redblox.randomizeNumber(0,answers.length-1)], this, true);
					}
					
					
//					// Cut Trees
					var _terrain:Object;
					
					try
					{
						_terrain = map.terrain[Math.floor(x / map.blitSize)][Math.floor(y / map.blitSize)];
						if (_terrain && _terrain.state == TerrainState.NORMAL &&_terrain.state == TerrainState.BURNT && _terrain.type != TerrainType.BUILDING)
						{
							_terrain.state = TerrainState.CUT;
							_terrain.layerID = _terrain.originalLayerID - map.subLayerID[_terrain.type];
							map.addToTreesThatAreUpdated(_terrain);
							for (var k:int = -2; k < 2; k++)
							{
								for (var l:int = -2; l < 2; l++)
								{
									map.blitIt(map.terrain[_terrain.i+k][_terrain.j+l], false);
								}
							}
							//mapRoom.terrainCounter(_terrain);
						}
						
						_terrain = map.terrain[Math.ceil(x / map.blitSize)][Math.ceil(y / map.blitSize)];
						if (_terrain && _terrain.state == TerrainState.NORMAL &&_terrain.state == TerrainState.BURNT && _terrain.type != TerrainType.BUILDING)
						{
							_terrain.state = TerrainState.CUT;
							_terrain.layerID = _terrain.originalLayerID - map.subLayerID[_terrain.type];
							map.addToTreesThatAreUpdated(_terrain);
							for (var k:int = -2; k < 2; k++)
							{
								for (var l:int = -2; l < 2; l++)
								{
									map.blitIt(map.terrain[_terrain.i+k][_terrain.j+l], false);
								}
							}
							//mapRoom.terrainCounter(_terrain);
						}
						
						_terrain = map.terrain[Math.floor(x / map.blitSize)][Math.ceil(y / map.blitSize)];
						if (_terrain && _terrain.state == TerrainState.NORMAL &&_terrain.state == TerrainState.BURNT && _terrain.type != TerrainType.BUILDING)
						{
							_terrain.state = TerrainState.CUT;
							_terrain.layerID = _terrain.originalLayerID - map.subLayerID[_terrain.type];
							map.addToTreesThatAreUpdated(_terrain);
							for (var k:int = -2; k < 2; k++)
							{
								for (var l:int = -2; l < 2; l++)
								{
									map.blitIt(map.terrain[_terrain.i+k][_terrain.j+l], false);
								}
							}
							//mapRoom.terrainCounter(_terrain);
						}
						
						_terrain = map.terrain[Math.ceil(x / map.blitSize)][Math.floor(y / map.blitSize)];
						if (_terrain &&_terrain.state == TerrainState.NORMAL &&_terrain.state == TerrainState.BURNT && _terrain.type != TerrainType.BUILDING)
						{
							_terrain.state = TerrainState.CUT;
							_terrain.layerID = _terrain.originalLayerID - map.subLayerID[_terrain.type];
							map.addToTreesThatAreUpdated(_terrain);
							for (var k:int = -2; k < 2; k++)
							{
								for (var l:int = -2; l < 2; l++)
								{
									map.blitIt(map.terrain[_terrain.i+k][_terrain.j+l], false);
								}
							}
							//mapRoom.terrainCounter(_terrain);
						}
					}
					catch (e:Error)
					{}
				}
			}
		}
		
		/**
		 * 
		 * @param	unit
		 */
		override protected function arrivedAtDestination ():void
		{
			//trace("\nUnit.arrivedAtDestination();");
			progress = 0;
			linePath.points = new Array();
			map.lineLayer.removeChild(targetPoint);
			clearLine();
			if (state == UnitState.CUT)
			{
				callout("Done cutting \nthis fire line!", this, true, false);
			}
			else if (state == UnitState.NORMAL)
			{
				callout("Done cutting \nthis fire line!", this, true, false);				
			}
		} 
		
		
		/**
		 * 
		 * @param	_terrain
		 */
		override public function sendUpdatedUnit(_forceSend:Boolean = false):void 
		{
			if (isMyUnit && (sendCounter >= 5 || _forceSend))
			{
				sendCounter = 0;
				var gameData:Object = new Object();
				gameData.action = GameDataType.UNIT_NEWLINEPATHPOINTS;
				gameData.userID = PlayerIOConnection.myUserID;
				
				gameData.points = linePath.points;
				gameData.speed = speed;
				gameData.speedOnPath = speedOnPath;
				gameData.progress = progress;
				gameData.state = state;
				gameData.speedVariantBasedOnTerrain = speedVariantBasedOnTerrain;
				//				data.isOffScreen = isOffScreen;
				//				data.isEnteringMap = isEnteringMap;
				gameData.id = id;
				gameData.type = type;
				gameData.oldLength = oldLength;
				gameData.oldProgress = oldProgress;
				gameData.x = x;
				gameData.y = y;
				gameData.rotation = unitIcon.rotation;
				gameData.targetPointRotation = targetPoint.rotation;
				gameData.pointName = pointName;
				gameData.playerName = PlayerIOConnection.playerName;
				gameData.isNewLinepath = isNewLinepath;				
				isNewLinepath = false;
				
				map.sendDataToPlayerIO(gameData);
			}
			sendCounter++;
		}
		
		override public function processUnit(_data:Object):void
		{
			//			if (map.isSetupComplete)
			if (true)
			{
				var tempPoints:Array = new Array();
				for (var i:int = 0; i < _data.points.length; i++)
				{
					tempPoints.push(new Point(_data.points[i].x, _data.points[i].y))
				}
				linePath.points = tempPoints;
				speed = _data.speed;
				speedOnPath = _data.speedOnPath;
				progress = _data.progress;
				oldLength = _data.oldLength;
				oldProgress = _data.oldProgress;
				unitIcon.rotation = _data.rotation;
				targetPoint.rotation = _data.targetPointRotation;
				speedVariantBasedOnTerrain = _data.speedVariantBasedOnTerrain;
				state = _data.state;
				
				if (redblox.isActive && _data.isNewLinepath) SoundAS.play((state == UnitState.CUT)? "CutTeam" : "Run",0.6).fadeTo(0,4000);
					
					
				if (state == UnitState.NORMAL) 
				{
					unitIcon.currentFrame = 1;
				}
				else
				{
					unitIcon.currentFrame = 0;
				}

				if (linePath.points.length > 0)
				{
					targetPoint.x = linePath.points[linePath.points.length-1].x;
					targetPoint.y = linePath.points[linePath.points.length-1].y;
					map.lineLayer.addChild(targetPoint);
				}
				else
				{
					if (map.lineLayer.contains(targetPoint)) map.lineLayer.removeChild(targetPoint);
				}
				//this.img.icon.visible = _data.visible;
				udpatePlayerName(_data.playerName);
				userID = _data.userID;
				updateSpeedAndProgress();
				//map.lineLayer.addChild(targetPoint);
				showPointType(_data.pointName);
				updatePointTypeByTerrain();
				clearLine();
				this.x = _data.x;
				this.y = _data.y;
				drawLineCounter = 10;
				drawLine();
				isDragAndDrop = false;
				this.visible = true;
				if (map.unitLayer.contains(this) == false) map.unitLayer.addChildAt(this, 0);

				//if (tempPoints.length > 0) TweenMax.to(endPoint, 0, { x:tempPoints[tempPoints.length - 1].x, y: tempPoints[tempPoints.length - 1].y, autoAlpha:1 } );
				
				if (targetUnit) // check to see if a helicopter is coming to pick him up
				{
					targetUnit.disable(false);
					targetUnit.targetUnit = null;
					targetUnit = null;
				}
			}
		}
		
		override protected function onMouseUpUnit():void
		{
			if (isDragAndDrop) 
			{
				callout("Ready to cut a fire \nline! I'm best used to \ncut GRASS & BRUSH.", this, true, false);
				map.unitLayer.addChildAt(this,0);
			}
			super.onMouseUpUnit();
			map.panelUnitMenu.show(containerMenu, this, "Cut Team");
//			map.soundFX["CutTeam"].play();
		}
		
		override protected function onMouseUpTargetPoint():void
		{
			super.onMouseUpTargetPoint();
			map.panelUnitMenu.show(containerMenu, this, "Cut Team");
		}
		
		override protected function onMouseDownUnit():void
		{
			if (redblox.isActive) SoundAS.play((state == UnitState.CUT)? "CutTeam" : "Run",0.6).fadeTo(0,4000);
			super.onMouseDownUnit();
			speedVariantBasedOnTerrain = 1;

			if (targetUnit) // check to see if a helicopter is coming to pick him up
			{
				targetUnit.disable(false);
				targetUnit.targetUnit = null;
				targetUnit = null;
			}
			
		}
		
		override protected function onMouseMoveUnit():void
		{
			tappedOnce = false;
			super.onMouseMoveUnit();
		}		
		
		
		override public function dispose():void
		{
			
			super.dispose();
		}
	}
}