package units
{
	import com.greensock.TweenMax;
	import com.greensock.motionPaths.LinePath2D;
	import com.greensock.plugins.BezierPlugin;
	
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.geom.Point;
	
	import maps.Map;
	
	import starling.animation.IAnimatable;
	import starling.core.Starling;
	import starling.display.Image;
	import starling.display.MovieClip;
	import starling.display.Sprite;
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.textures.Texture;
	import starling.utils.Line;
	
	public class Unit extends Sprite implements IAnimatable
	{
		private var unitIcon:MovieClip;
		private var touchUnit:Touch;
		private var isCollect:Boolean = false;
		
		private var linePath:LinePath2D;
		private var linePathTailEnd:LinePath2D;
		
		private var speed:Number = 1.5;
		private var progress:Number = 0;
		private var maxDistance:Number = 200;
		private var oldLength:Number = 0;
		private var speedOnPath:Number = 0;
		private var timeScale:Number = 1;
		private var oldProgress:Number = 0;
		private var targetRotation:Number = 0;
		private var dr:Number = 0;
		private var ar:Number = 0;
		private var vr:Number = 0;
		private var rotationDamping:Number = 0.6;
		private var rotationSpring:Number = 0.1;
		private var lines:Sprite;
		private var gap:int = 20;
		private var linePointArray:Array;
		private var numDots:int = 100;
		private var map:Map;
		private var state:String;
		private var anchorPoint:Image;
		private var targetUnit:Sprite;
		private var hasAnchorPoint:Boolean;
		private var hasAnchorPointDrop:Boolean;
		
		private var myBez:BezierCurve = new BezierCurve();
		
		private var staticPoints:Array = new Array();
		
		public function Unit(_lines:Sprite, _map:Map)
		{
			super();
			lines = _lines;
			map = _map;
			this.addEventListener(starling.events.Event.ADDED_TO_STAGE, onAddedToStage);
		}
		
		private function onAddedToStage():void
		{
			this.removeEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
			initializeUnit();
		}
		
		protected function initializeUnit():void
		{
			// To activate function advanceTime
			Starling.juggler.add(this); 
			
			// Add dots to use for line path
			linePointArray = new Array();
			for (var i:int = 0; i < numDots; i++)
			{
				linePointArray.push(new Image(Assets.getAtlas().getTexture("dot")));
			}
			
			
			// Add greensock LinePath2D for unit to follow
			linePath = new LinePath2D();
			linePathTailEnd = new LinePath2D();
			
			this.addEventListener(TouchEvent.TOUCH, onTouchUnit);
			
			//this.addChild(myBez);
		}
		
		protected function initializeUnitIcon(name:String):void
		{
			
			unitIcon = new MovieClip(Assets.getAtlas().getTextures(name + "_"), 10);
			unitIcon.alignPivot();
			this.addChild(unitIcon);
			
			starling.core.Starling.juggler.add(unitIcon);
		}
		
		public function advanceTime(passedTime:Number):void
		{
			
			if (isCollect) 
			{
				// Add new linePath points
				var point:Point = new Point((touchUnit.globalX - map.x)/map.scaleX,(touchUnit.globalY - map.y)/ map.scaleY);
//				if (linePath.points.length == 0)
				if (staticPoints.length == 0)
				{
					progress = 0;
					//linePath.appendPoint(new Point(x,y));
					staticPoints.push(new Point(x,y));
					return;
				}
				else
				{
//					var lastPoint:Point = linePath.points[linePath.points.length-1];
					var lastPoint:Point = staticPoints[staticPoints.length-1];
					if (linePath.points.length != 1)
					{
						if (Point.distance(point, lastPoint) >= maxDistance)
						{
							//var bezTween:TweenMax = 
							//	BezierPlugin.;
							//linePath.appendPoint(point);
							
							staticPoints.push(point);
							
							//linePathTailEnd.points = myBez.grabPoints(staticPoints);
							linePath.points = myBez.grabPoints(staticPoints);
							linePath.progress = progress;
							//linePathTailEnd.progress = progress;
							
							//calculatePoints(lastPoint, point, linePath);
							updateSpeedAndProgress();
						}
						else
						{
							var beforeLastPoint:Point = staticPoints[staticPoints.length-2];
							if (beforeLastPoint)
							{
								var tempStaticPoints:Array = new Array()
								tempStaticPoints.push(beforeLastPoint);
								tempStaticPoints.push(lastPoint);
								tempStaticPoints.push(point);
	//							progress = linePath.progress;
	//							updateSpeedAndProgress();
								linePathTailEnd.points = myBez.grabPoints(tempStaticPoints);
							}
//							linePath.points = myBez.grabPoints(tempStaticPoints);
//							linePath.progress = progress;
//							updateSpeedAndProgress();
							
							//calculatePoints(lastPoint, point, linePath);
							//updateSpeedAndProgress();
						}
					}
					else
					{
						linePath.appendPoint(point);
						updateSpeedAndProgress();
					}
				}
			}
			
			// Draw series of dots along unit's linePath
			drawLine();
			
			// Advance unit's progress along linePath
			progress += speedOnPath * TweenMax.globalTimeScale * passedTime*30;
			if (progress > 1) progress = 1;
			if (linePath.points.length > 1) 
			{
				linePath.renderObjectAt(this, progress, false, 0);
				targetRotation = linePath.angle;
			}
			
			oldProgress = progress;
		}
		
		/**
		 * 
		 * @param	unit
		 */
		public function arrivedAtDestination ():void
		{

		} 
		
		public function drawLine():void
		{
			if (linePath.points.length > 0)
			{
//				var tempLinePath:LinePath2D
				var points:Number = linePath.totalLength / gap;
				var num:Number = 1 / points;
				var angle:Number;
				
				//spread the points evenly  
				var p:Number = 0;
				var point:Point;
				var prevPoint:Point;
				
				var currentDot:int = 0;
				
//				var upToPoints:Array = linePath.getAllPointsUpToProgress(progress);
//				var afterPoints:Array = linePath.getAllPointsAfterProgress(progress);
//				
//				for (var i:int = 0; i < upToPoints.length; i++)
//				{
//					if (lines.contains(linePointArray[i])) lines.removeChild(linePointArray[i]);
//				}
//				
//				for (var i:int = 0; i < afterPoints.length; i++)
//				{
////					if (!lines.contains(linePointArray[i])) 
////					{
//						linePointArray[i].x = afterPoints[i].x - 3;
//						linePointArray[i].y = afterPoints[i].y - 3;
//						lines.addChild(linePointArray[i]);
////					}
//				}
				
				while (p < 1)
				{
					
					//check p against progress so we don't draw the dashes/dots the plane has moved past already  
					if (p > progress && linePointArray[currentDot])
					{
						point = linePath.getPointAtProgress(p);
						linePointArray[currentDot].x = point.x - 3;
						linePointArray[currentDot].y = point.y - 3;
						if (!lines.contains(linePointArray[currentDot])) lines.addChild(linePointArray[currentDot]);
						currentDot++;
						prevPoint = point;
					}
					p += num;
				}
				
				var points:Number = linePathTailEnd.totalLength / gap;
				var num:Number = 1 / points;
				var angle:Number;
				
				//spread the points evenly  
				var p:Number = 0;
				var point:Point;
				var prevPoint:Point;
				
				
				while (p < 1)
				{
					
					//check p against progress so we don't draw the dashes/dots the plane has moved past already  
					if (p > progress && linePointArray[currentDot])
					{
						point = linePathTailEnd.getPointAtProgress(p);
						linePointArray[currentDot].x = point.x - 3;
						linePointArray[currentDot].y = point.y - 3;
						if (!lines.contains(linePointArray[currentDot])) lines.addChild(linePointArray[currentDot]);
						currentDot++;
						prevPoint = point;
					}
					p += num;
				}
				
				for (var i:int = currentDot; i < numDots; i++)
				{
					if (lines.contains(linePointArray[i])) lines.removeChild(linePointArray[i]);
				}
			}
		}
		
		private function clearLine():void
		{
			for (var i:int = 0; i < numDots; i++)
			{
				if (lines.contains(linePointArray[i])) lines.removeChild(linePointArray[i]);
			}
		}
		
		private function updateSpeedAndProgress():void
		{
			if (speed != 0)
			{
				var iterations:int = linePath.totalLength/(speed * TweenMax.globalTimeScale);  
				speedOnPath = 1 / iterations;
				
				if (oldLength != 0 && linePath.totalLength - oldLength > 10) {  
					progress = (oldLength * oldProgress)/linePath.totalLength;  
					linePath.renderObjectAt(this, progress);
					linePath.progress = progress;
				}  
				oldLength = linePath.totalLength;  
				oldProgress = progress;  
			}
		}
		
		private function calculatePoints(_point1:Point, _point2:Point, _linePath:LinePath2D):void
		{
			if (_point1 && _point2 && _linePath && _linePath.points.length < numDots)
			{
				var i:int; // for loop
				
				var o:Number = _point2.y - _point1.y; // Opposite Side
				var o_neg:Number = (o > 0) ? 1 : -1;
				var a:Number = _point2.x - _point1.x; // Adjascent Side
				var a_neg:Number = (a > 0) ? 1 : -1;
				var h:Number = Point.distance(_point1, _point2); // Hypotenuse Side
				var angle:Number = Math.abs(Math.asin(o / h)); //(Math.PI/180);
				
				var new_o:Number = gap * Math.sin(angle) * o_neg;
				var new_a:Number = gap * Math.cos(angle) * a_neg;
//				var new_o:Number = maxDistance * Math.sin(angle) * o_neg;
//				var new_a:Number = maxDistance * Math.cos(angle) * a_neg;
				
//				var numNewDots:Number = Math.ceil(h / maxDistance);
				var numNewDots:Number = Math.ceil(h / gap);
				
				for (i = 1; i < numNewDots; i++)
				{
					var nextPoint:Point = new Point(Math.round(_point1.x + (new_a * i)), Math.round(_point1.y + (new_o * i)));
					
					_linePath.appendPoint(nextPoint);
					// This is for special unitLayer.units that have a limit to the distance they can go (ie, Hoser, Bomber)
					if (_linePath.points.length >= numDots)//(hasReachedLimit(_linePath))
					{
//						endPoint.x = nextPoint.x;
//						endPoint.y = nextPoint.y;
						trace("hasReachedLimit");
						onMouseUpUnit();
						break;
					}
				}
			}			
		}
		
		private function fixLinePath():void
		{
			var prevPoint:Point = linePath.points[0]; 
			var maxAngle:Number = 10;
			var currentPoint:Point;
			var lastAngle:Number = 0;
			var currentAngle:Number = 0;
			
			for (var i:int = 1; i < linePath.points.length; i++)
			{
				currentPoint = linePath.points[i]
				currentAngle = redblox.point_direction(prevPoint.x, prevPoint.y, currentPoint.x, currentPoint.y);
				if (currentAngle - lastAngle > maxAngle) linePath.points.splice(i,1);
				lastAngle = currentAngle;
			}
		}
		
		private function onTouchUnit(e:TouchEvent):void
		{
			touchUnit = e.getTouch(this);
			
			
			if (touchUnit)
			{
				if (touchUnit.phase == TouchPhase.BEGAN)
				{
					trace("Unit.onTouchUnit(): " + (touchUnit.target == this));
					trace(" > target: " + touchUnit.target);
					trace(" > phase: " + touchUnit.phase);
					trace(" > onMouseDownUnit()");
					onMouseDownUnit();
				}
				else if (touchUnit.phase == TouchPhase.ENDED)
				{
					trace("Unit.onTouchUnit(): " + (touchUnit.target == this));
					trace(" > target: " + touchUnit.target);
					trace(" > phase: " + touchUnit.phase);
					trace(" > onMouseUpUnit()");
					onMouseUpUnit();	
				}
				else if (touchUnit.phase == TouchPhase.MOVED)
				{
					onMouseMoveUnit();
				}
			}
		}
		
		protected function rotateUnitAlongPath()
		{
			// Rotate unit smoothly
			if (targetRotation > unitIcon.rotation * 180 / Math.PI + 180) targetRotation -= 360;
			if (targetRotation < unitIcon.rotation * 180 / Math.PI - 180) targetRotation += 360;
			dr = (targetRotation * Math.PI / 180) - unitIcon.rotation;
			ar = dr * rotationSpring;  
			vr += ar;  
			vr *= rotationDamping;
			unitIcon.rotation += vr;
		}
		
		protected function onMouseMoveUnit():void
		{
		}
		
		protected function onMouseUpUnit():void
		{
			trace("Unit.onMouseUpUnit()");
			isCollect = false;
		}
		
		protected function onMouseDownUnit():void
		{
			clearLine();
			isCollect = true;
			progress = 0;
			linePath.points.length = 0;
			linePath.points = new Array();
			staticPoints = new Array();
			linePathTailEnd.points = new Array();
		}
	}
}