package units
{
	import com.greensock.TweenMax;
	import com.greensock.motionPaths.LinePath2D;
	
	import flash.geom.Point;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	import flash.text.engine.ElementFormat;
	import flash.utils.Dictionary;
	
	import feathers.controls.Callout;
	import feathers.controls.Label;
	import feathers.controls.ScrollContainer;
	import feathers.controls.text.TextFieldTextRenderer;
	import feathers.core.ITextRenderer;
	import feathers.core.PopUpManager;
	import feathers.controls.LayoutGroup;
	
	import maps.GameDataType;
	import maps.Map;
	import maps.Terrain;
	import maps.TerrainState;
	import maps.TerrainType;
	
	import org.gestouch.events.GestureEvent;
	import org.gestouch.gestures.TapGesture;
	
	import starling.animation.IAnimatable;
	import starling.core.Starling;
	import starling.display.DisplayObject;
	import starling.display.Graphics;
	import starling.display.Image;
	import starling.display.MovieClip;
	import starling.display.Sprite;
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.filters.BlurFilter;
	import starling.filters.ColorMatrixFilter;
	
	public class Unit extends Sprite implements IAnimatable
	{
		public var unitIcon:MovieClip;
		public var unitIconMini:MovieClip;
		protected var unitIconCallout:MovieClip;
		protected var unitIconShadow:MovieClip;
		public var unitIconCollision:MovieClip;
		public var iconUnitContainer:LayoutGroup = new LayoutGroup();

		protected var touchUnit:Touch;
		protected var touchTargetPoint:Touch;
		protected var isCollect:Boolean = false;
		
		protected var linePath:LinePath2D;
		protected var linePathTailEnd:LinePath2D;
		
		protected var speed:Number = 1;
		protected var speedVariantBasedOnTerrain:Number = 1;
		protected var progress:Number = 0;
		protected var maxDistance:Number = 30;
		protected var oldLength:Number = 0;
		protected var speedOnPath:Number = 0;
		protected var timeScale:Number = 1;
		protected var oldProgress:Number = 0;
		protected var targetRotation:Number = 0;
		protected var currentRotation:Number = 0;
		protected var dr:Number = 0;
		protected var ar:Number = 0;
		protected var vr:Number = 0;
		protected var rotationDamping:Number = 0.6;
		protected var rotationSpring:Number = 0.1;
		protected var gap:int = 20;
		protected var linePointArray:Array;
		protected var numDots:int = 100;
		protected var map:Map;
		protected var anchorPoint:Image;
		protected var hasAnchorPoint:Boolean;
		protected var hasAnchorPointDrop:Boolean;
		
		protected var UnitSpeeds:Object;
		
		protected var terrainUnderUnit:Object;
		protected var terrainUnderUnitPrevious:Object;
		
		protected var targetPoint:Sprite = new Sprite();
		protected var pointType:Dictionary = new Dictionary();
		
		protected var myBez:BezierCurve = new BezierCurve();
		
		protected var hasArrivedAtDestination:Boolean = false;
		
		public var id:int;
		public var type:int;
		public var state:int;
		protected var sendCounter:int = 0;
		protected var pointName:String = "";
		protected var drawLineCounter:int = 0;
		public var targetUnit:Unit;
		//protected var targetUnit:Sprite;
		
		public var isMyUnit:Boolean = true;
		public var playerName:String = "";
		public var userID:String = "";
		public var isOnHelicopter:Boolean = false;
		public var playerNameLabel:Label = new Label();
		public var countdownLabel:Label = new Label();
		
		public var newLinePathPoints:Array;
		
		protected var tapGesture:TapGesture;
		
		public var isDragAndDrop:Boolean = true;
		public var namePopup:Sprite;
		public var countdownPopup:Sprite;
		private var backgroundSkin2:Image;
		private var countdownBackgroundSkin:Image;
		public var calloutMessage:Callout;
//		public var calloutLocator:Callout;
		public var calloutLabel:Label = new Label();
		public var originSprite:Sprite = new Sprite();
		public var originSprite2:Sprite = new Sprite();
		
		public var unitMenu:Callout;
		public var containerMenu:ScrollContainer = new ScrollContainer(); 
		
		public var isEnabled:Boolean = false;
		
		public var isNewLinepath:Boolean = false;
		private var calloutContainer:Sprite = new Sprite();
		
		public var previousCheckIfUnitIsInFrame:Boolean = false;
		private var currentCheckIfUnitIsInFrame:Boolean;
		public var isGroundUnitNearBy:Boolean = false;
		
		
		
		public function Unit(_map:Map, _isMyUnit:Boolean, _id:int)
		{
			super();
			map = _map;
			isMyUnit = _isMyUnit;
			id = _id;
			
			this.visible = isMyUnit;
				
			this.addEventListener(starling.events.Event.ADDED_TO_STAGE, onAddedToStage);
		}
		
		protected function onAddedToStage():void
		{
			this.removeEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
//			initializeUnit();
			initializeOnce();
		}
		
		protected function initializeOnce():void
		{
			// Add dots to use for line path
			linePointArray = new Array();
			for (var i:int = 0; i < numDots; i++)
			{
				linePointArray.push(new Image(Assets.getAtlas().getTexture("dot")));
				map.lineLayer.addChild(linePointArray[i]);
				linePointArray[i].visible = false;
			}
			
			clearLine();
			
			addNewPointType("targetNormal");
			showPointType("targetNormal");
			
			originSprite = new Sprite();
			addChild(originSprite);
			addChild(originSprite2);

			
//			tapGesture = new TapGesture(unitIcon);
//			tapGesture.numTouchesRequired = 1;
//			tapGesture.addEventListener(GestureEvent.GESTURE_RECOGNIZED, onDoubleTapUnitIcon);
		}
		
		protected function onDoubleTapUnitIcon(event:GestureEvent):void
		{
//			showUnitMenu();
//			trace("Unit.onTapUnitIcon");
//			if (state == UnitState.NORMAL) 
//			{
//				state = UnitState.CUT;
//				unitIcon.currentFrame = 1;
//			}
//			else
//			{
//				state = UnitState.NORMAL;
//				unitIcon.currentFrame = 2;
//			}
		}
		
		public function initializeUnit():void
		{
			
			unitIcon.rotation = 0;
			targetRotation = 0;
			
			// To activate function advanceTime
			enable();//Starling.juggler.add(this); 
			
			// Add greensock LinePath2D for unit to follow
			linePath = new LinePath2D();
			linePathTailEnd = new LinePath2D();
			
			if (isMyUnit || type == UnitType.CIVILIAN)
			{
				this.addEventListener(TouchEvent.TOUCH, onTouchUnit);
				targetPoint.addEventListener(TouchEvent.TOUCH, onTouchTargetPoint);
			}
			else
			{
				if (this.hasEventListener(TouchEvent.TOUCH))
					this.removeEventListener(TouchEvent.TOUCH, onTouchUnit);
				if (targetPoint.hasEventListener(TouchEvent.TOUCH))
					targetPoint.removeEventListener(TouchEvent.TOUCH, onTouchTargetPoint);
			}
			
			playerNameLabel.text = playerName;
			playerNameLabel.touchable = false;
			if (namePopup) namePopup.touchable = false;
			//playerNameLabel.touchable = false;
			//addChild(playerNameLabel);
			if (map.lineLayer.contains(targetPoint)) map.lineLayer.removeChild(targetPoint);
			
			if (namePopup == undefined)	
			{
				namePopup = new Sprite();
				namePopup.touchable = false;
				namePopup.scaleX = namePopup.scaleY = 0.65;
				backgroundSkin2 = new Image(Assets.getAtlas().getTexture("greyPanel"));
				backgroundSkin2.touchable = false;
				namePopup.addChild(backgroundSkin2);
				namePopup.addChild(playerNameLabel);
				if (type != UnitType.CIVILIAN && type != UnitType.ZOMBIE) addChild(namePopup);
			}
			//playerNameLabel.width = stage.stageWidth/4 - 10;
			udpatePlayerName(playerName);
			
			
			countdownLabel.text = "";
			countdownLabel.touchable = false;
			if (countdownPopup) countdownPopup.touchable = false;
			//playerNameLabel.touchable = false;
			//addChild(playerNameLabel);
			if (countdownPopup == undefined)	
			{
				countdownPopup = new Sprite();
				countdownPopup.touchable = false;
//				countdownPopup.scaleX = countdownPopup.scaleY = 0.75;
				countdownBackgroundSkin = new Image(Assets.getAtlas().getTexture("greyPanel"));
				countdownBackgroundSkin.touchable = false;
				countdownPopup.addChild(countdownBackgroundSkin);
				countdownPopup.addChild(countdownLabel);
				//if (type != UnitType.CIVILIAN) addChild(namePopup);
			}
			//playerNameLabel.width = stage.stageWidth/4 - 10;
			
			
			//var box:Graphics = new Graphics(
			
			
			//trace("done namePopup");
		}
		
		public function udpatePlayerName(_playerName:String):void
		{
			playerName = _playerName;
			playerNameLabel.text = playerName;
			playerNameLabel.validate();
			playerNameLabel.x = -playerNameLabel.width/2;
			playerNameLabel.y = -playerNameLabel.height/2;
			backgroundSkin2.width = playerNameLabel.width + 10;
			backgroundSkin2.height = playerNameLabel.height + 10;
			backgroundSkin2.x = -backgroundSkin2.width/2;
			backgroundSkin2.y = -backgroundSkin2.height/2;
		}
		
		public function udpateCountdown(_countdown:String):void
		{
			if (countdownLabel.text != _countdown)
			{
				countdownLabel.text = _countdown;
				countdownLabel.validate();
				countdownLabel.x = -countdownLabel.width/2;
				countdownLabel.y = -countdownLabel.height/2;
				countdownBackgroundSkin.width = countdownLabel.width + 10;
				countdownBackgroundSkin.height = countdownLabel.height + 10;
				countdownBackgroundSkin.x = -countdownBackgroundSkin.width/2;
				countdownBackgroundSkin.y = -countdownBackgroundSkin.height/2;
			}
		}
		
		public function enable():void
		{
			Starling.juggler.add(this);
			isEnabled = true;
			//map.unitLayer.addChild(this);

		}
		
		public function disable(_removeJuggler:Boolean = true):void
		{
			isEnabled = false;
			if (_removeJuggler) Starling.juggler.remove(this);
			clearLine();
			progress = 0;
			linePath.points = new Array();
//			if (map.lineLayer.contains(targetPoint)) map.lineLayer.removeChild(targetPoint);
			targetPoint.removeFromParent();
		}
		
		public function reset():void
		{
			isEnabled = false;
			userID = "";
			playerName = "";
			state = UnitState.NORMAL;
			Starling.juggler.remove(this);
			clearLine();
			if (unitIcon) unitIcon.rotation = 0;
			targetRotation = 0;
			targetPoint.rotation = 0;
			progress = 0;
			scaleX = scaleY = 1;
			if (linePath) linePath.points = new Array();
//			if (map && map.lineLayer.contains(targetPoint)) map.lineLayer.removeChild(targetPoint);
			targetPoint.removeFromParent();
			isDragAndDrop = true;
			x = -500;
			y = -500;
			if (unitIconCollision) unitIconCollision.visible = false;
//			if (parent) parent.removeChild(this);
			this.removeFromParent();
			
			if (calloutMessage) calloutMessage.close();

			for (var i:int = 0; i < map.unitList.length; i++)
			{
				if (map.unitList[i] == this)
				{
					map.unitList.splice(i,1);
					break;
				}
			}
			
		}
		
		protected function addNewPointType(name:String, isMovieClip:Boolean = false, frameRate:Number = 10):void
		{
			if (isMovieClip)
			{
				pointType[name] = new MovieClip(Assets.getAtlas().getTextures(name + "_"), frameRate);
				pointType[name].x = -pointType[name].width/2;
				pointType[name].y = -pointType[name].height/2;
				starling.core.Starling.juggler.add(pointType[name]);
			}
			else
			{
				pointType[name] = new Image(Assets.getAtlas().getTexture(name));
				pointType[name].x = -pointType[name].width/2;
				pointType[name].y = -pointType[name].height/2;
			}
			
		}
		
		protected function showPointType(name:String):void
		{
			if (targetPoint.numChildren > 0) targetPoint.removeChildAt(0);
			if (pointType[name]) targetPoint.addChild(pointType[name]);
			pointName = name;
		}
		
		protected function updatePointTypeByTerrain():void
		{
			try
			{				
				var terrainType:Number = map.terrain[Math.round(targetPoint.x/map.blitSize)][Math.round(targetPoint.y/map.blitSize)].type;
				if (terrainType == TerrainType.WATER_DEEP)
				{
					showPointType("targetHelicopterWaterPump");
				}
				else
				{
					showPointType("targetNormal");
				}
			}
			catch(e:Error)
			{
				showPointType("targetNormal");
			}
		}
		
		public function toggleNight():void
		{
			
		}
		
		protected function initializeUnitIcon(name:String, isJuggle:Boolean = true, speed:int = 10):void
		{
			trace("initializeUnitIcon: " + name);
			if (unitIconCollision) unitIconCollision.removeFromParent(true);
			unitIconCollision = new MovieClip(Assets.getAtlas().getTextures("unitIconCollision_"), speed);
			unitIconCollision.alignPivot();
			unitIconCollision.visible = false;
			//iconUnitContainer.addChild(unitIconCollision);
			starling.core.Starling.juggler.add(unitIconCollision);
			
			if (unitIcon) unitIcon.removeFromParent(true);
			unitIcon = new MovieClip(Assets.getAtlas().getTextures(name + "_"), speed);
			unitIcon.alignPivot();
			
			if (unitIconMini) unitIconMini.removeFromParent(true);
			unitIconMini = new MovieClip(Assets.getAtlas().getTextures(name + "Mini_"), speed);
			unitIconMini.alignPivot();
			
			if (unitIconCallout) unitIconCallout.removeFromParent(true);
			unitIconCallout = new MovieClip(Assets.getAtlas().getTextures(name + "_"), speed);
			unitIconCallout.alignPivot();
//			unitIconCallout.alpha = 0;
//			unitIconCallout.touchable = false;
			unitIconCallout.scaleX = unitIconCallout.scaleY = 0.5;
			
			if (isJuggle) starling.core.Starling.juggler.add(unitIcon);
			if (isJuggle) starling.core.Starling.juggler.add(unitIconMini);
			if (isJuggle) starling.core.Starling.juggler.add(unitIconCallout);
			
			if (unitIconShadow) unitIconShadow.removeFromParent(true);
			unitIconShadow = new MovieClip(Assets.getAtlas().getTextures(name + "_"), speed);
			unitIconShadow.alignPivot();
			iconUnitContainer.addChild(unitIconShadow);
			unitIconShadow.x += 10;
			unitIconShadow.y += 10;
			unitIconShadow.color = 0x000000;
			unitIconShadow.alpha = 0.3;
			iconUnitContainer.addChild(unitIcon);
			
			this.addChild(iconUnitContainer);
			
			if (isJuggle) starling.core.Starling.juggler.add(unitIconShadow);
		}
		
		public function advanceTime(passedTime:Number):void
		{
			updateUnitOnNewTerrain();
			
			if (isDragAndDrop || state == UnitState.DEAD)
			{
				return;
			}
			
			if (isCollect) 
			{
				// Add new linePath points
				var point:Point = new Point((touchUnit.globalX - map.x)/map.scaleX,(touchUnit.globalY - map.y)/ map.scaleY);
				targetPoint.x = point.x;
				targetPoint.y = point.y;
				updatePointTypeByTerrain();
				map.updateMinimap(targetPoint);

				var lastPoint:Point = linePath.points[linePath.points.length-1];
				
				if (lastPoint && Point.distance(point, lastPoint) >= maxDistance)
				{
					calculatePoints(lastPoint, point, linePath);
					updateSpeedAndProgress();
					//map.updateMinimapTransparency(-0.05);
				}
				else
				{
//					map.updateMinimapTransparency(+0.05);
				}
				targetPoint.rotation = linePath.getAngleAtProgress(1);
				sendUpdatedUnit();
				drawLine();
			}
			
			
			// Follow linePath if it exists
			if (linePath.totalLength > 0)
			{
				// Draw series of dots along unit's linePath
				//drawLine();
				
				// Advance unit's progress along linePath
				progress += speedOnPath * TweenMax.globalTimeScale * passedTime*30;
				if (progress > 1) progress = 1;
				if (linePath.points.length > 1) 
				{
					linePath.renderObjectAt(this, progress, false, 0);
					targetRotation = linePath.angle;
				}
				
				oldProgress = progress;
				
				if (progress == 1) arrivedAtDestination();
			}
			
			
//			originSprite2.x = x;
//			originSprite2.y = y;
			
//			this.scaleX = this.scaleY = Math.min(1, 1/map.scaleX);
			
			
		}
		
		protected function updateUnitOnNewTerrain():void
		{
			try
			{
				terrainUnderUnitPrevious = terrainUnderUnit;
				terrainUnderUnit = map.terrain[Math.round(x/map.blitSize)][Math.round(y/map.blitSize)];
			}catch(e:Error)
			{
				terrainUnderUnit = null;
			}
			
			if (terrainUnderUnitPrevious != terrainUnderUnit) drawLine();
			
			if (isMyUnit || (type == UnitType.CIVILIAN && isGroundUnitNearBy)) checkIfUnitIsInFrame();

		}
		
		/**
		 * 
		 * @param	unit
		 */
		protected function arrivedAtDestination ():void
		{
			//trace("\nUnit.arrivedAtDestination();");
			progress = 0;
			linePath.points = new Array();
			map.lineLayer.removeChild(targetPoint);
		} 
		
		protected function drawLine():void
		{
			if (linePath.points.length > 0)
			{
				//drawLineCounter = 0;
				var points:Number = linePath.totalLength / gap;
				var num:Number = 1 / points;
				var angle:Number;
				
				//spread the points evenly  
				var p:Number = 0;
				var point:Point;
				var prevPoint:Point;
				
				var currentDot:int = 0;
				
				var upToPoints:Array = linePath.getAllPointsUpToProgress(progress);
				var afterPoints:Array = linePath.getAllPointsAfterProgress(progress);
				
				for (var i:int = 0; i < upToPoints.length; i++)
				{
					linePointArray[currentDot].visible = false;
					currentDot++;
				}
				
				for (var i:int = 0; i < afterPoints.length - 1; i++)
				{
					if (linePointArray[currentDot])
					{
						if (!linePointArray[currentDot].visible) 
						{
							linePointArray[currentDot].x = afterPoints[i].x - 8;
							linePointArray[currentDot].y = afterPoints[i].y - 8;
							linePointArray[currentDot].visible = true;
							linePointArray[currentDot].alpha = 1;
							if (!isMyUnit) TweenMax.to(linePointArray[currentDot], 1, {alpha:0.4, delay:0.02 * currentDot, overwrite:true});
						}
						currentDot++;
					}
					else
					{
						break;
					}
					
				}
				
			}
			//drawLineCounter++;
		}
		
		/**
		 * 
		 * @param	_terrain
		 */
		public function sendUpdatedUnit(_forceSend:Boolean = false):void 
		{

		}
		
		public function processUnit(_data:Object):void
		{

		}
		
		protected function clearLine():void
		{
			if (linePointArray)
			{
				for (var i:int = 0; i < numDots; i++)
				{
					linePointArray[i].visible = false;
	//				if (map.lineLayer.contains(linePointArray[i])) map.lineLayer.removeChild(linePointArray[i]);
				}
			}
		}
		
		protected function updateSpeedAndProgress():void
		{
			if (speed != 0)
			{
				var iterations:int = linePath.totalLength/(speed);  
				speedOnPath = 1 / iterations;
				
				if (oldLength != 0 && linePath.totalLength - oldLength > 10) {  
					progress = (oldLength * oldProgress)/linePath.totalLength;  
					linePath.renderObjectAt(this, progress);
					linePath.progress = progress;
				}  
				oldLength = linePath.totalLength;  
				oldProgress = progress;  
			}
		}
		
		protected function calculatePoints(_point1:Point, _point2:Point, _linePath:LinePath2D):void
		{
			if (_point1 && _point2 && _linePath && _linePath.points.length < numDots)
			{
				var i:int; // for loop
				
				var o:Number = _point2.y - _point1.y; // Opposite Side
				var o_neg:Number = (o > 0) ? 1 : -1;
				var a:Number = _point2.x - _point1.x; // Adjascent Side
				var a_neg:Number = (a > 0) ? 1 : -1;
				var h:Number = Point.distance(_point1, _point2); // Hypotenuse Side
				var angle:Number = Math.abs(Math.asin(o / h)); //(Math.PI/180);
				
				var new_o:Number = gap * Math.sin(angle) * o_neg;
				var new_a:Number = gap * Math.cos(angle) * a_neg;
//				var new_o:Number = maxDistance * Math.sin(angle) * o_neg;
//				var new_a:Number = maxDistance * Math.cos(angle) * a_neg;
				
//				var numNewDots:Number = Math.ceil(h / maxDistance);
				var numNewDots:Number = Math.ceil(h / gap);
				
				for (i = 1; i < numNewDots; i++)
				{
					var nextPoint:Point = new Point(Math.round(_point1.x + (new_a * i)), Math.round(_point1.y + (new_o * i)));
					
					_linePath.appendPoint(nextPoint);
					// This is for special unitLayer.units that have a limit to the distance they can go (ie, Hoser, Bomber)
//					if (_linePath.points.length >= numDots * maxDistance/gap)//(hasReachedLimit(_linePath))
					if (_linePath.points.length >= numDots)//(hasReachedLimit(_linePath))
					{
//						endPoint.x = nextPoint.x;
//						endPoint.y = nextPoint.y;
//						trace("hasReachedLimit");
						onMouseUpUnit();
						break;
					}
				}
			}			
		}
		
		protected function fixLinePath():void
		{
			var prevPoint:Point = linePath.points[0]; 
			var maxAngle:Number = 10;
			var currentPoint:Point;
			var lastAngle:Number = 0;
			var currentAngle:Number = 0;
			
			for (var i:int = 1; i < linePath.points.length; i++)
			{
				currentPoint = linePath.points[i]
				currentAngle = redblox.point_direction(prevPoint.x, prevPoint.y, currentPoint.x, currentPoint.y);
				if (currentAngle - lastAngle > maxAngle) linePath.points.splice(i,1);
				lastAngle = currentAngle;
			}
		}
		
		protected function onTouchUnit(e:TouchEvent):void
		{
			touchUnit = e.getTouch(this);
			
			
			if (touchUnit)
			{
				if (touchUnit.phase == TouchPhase.BEGAN)
				{
//					trace("Unit.onTouchUnit(): " + (touchUnit.target == this));
//					trace(" > target: " + touchUnit.target);
//					trace(" > phase: " + touchUnit.phase);
//					trace(" > onMouseDownUnit()");
					onMouseDownUnit();
					map.resetIdle();
				}
				else if (touchUnit.phase == TouchPhase.ENDED)
				{
//					trace("Unit.onTouchUnit(): " + (touchUnit.target == this));
//					trace(" > target: " + touchUnit.target);
//					trace(" > phase: " + touchUnit.phase);
//					trace(" > onMouseUpUnit()");
					onMouseUpUnit();	
				}
				else if (touchUnit.phase == TouchPhase.MOVED)
				{
					onMouseMoveUnit();
				}
			}
		}

		public function callout(_message:String, _coord:DisplayObject, _useNewOrigin:Boolean = false, _autoDispose:Boolean = true):void
		{
			if (isMyUnit || type == UnitType.CIVILIAN)
			{
				if (_coord.stage)
				{
					trace("callout: " + _message);
//					this.parent.addChild(originSprite);	
//					this.parent.addChild(originSprite2);	
//					originSprite.addChild(unitIconCallout);
	
		//			calloutLabel = new Label()
					calloutLabel = new Label();
//					calloutLabel.textRendererProperties.elementFormat = new ElementFormat(map.theme.regularFontDescription, 18 * map.theme.scale, 0xFFFFFF);;
//					calloutLabel.textRendererFactory = function():ITextRenderer
//					{
//						var renderer:TextFieldTextRenderer = new TextFieldTextRenderer();
//						renderer.textFormat = new TextFormat("Source Sans Pro",24 * map.theme.scale, 0xFFFFFF,null,null,null,null,null,TextFormatAlign.CENTER);
////						renderer.embedFonts = true;
//						return renderer;
//					}
					calloutLabel.textRendererProperties.leading = 5;
//					calloutLabel.textRendererProperties.embedFonts = true;
					calloutLabel.textRendererProperties.textAlign = TextFormatAlign.CENTER;;
					//calloutLabel.textRendererProperties.color = 0xFF0000;
					
					calloutLabel.text = _message;
					//calloutContainer.addChild(calloutLabel);
					
					if (calloutMessage) calloutMessage.close();
					calloutMessage = Callout.show(calloutLabel, _coord, Callout.DIRECTION_UP, false);
					//calloutMessage.scaleX = calloutMessage.scaleY = 1;
					
					calloutMessage.bottomArrowSkin.removeFromParent();
					calloutMessage.topArrowSkin.removeFromParent();
					calloutMessage.leftArrowSkin.removeFromParent();
					calloutMessage.rightArrowSkin.removeFromParent();
//					calloutMessage = Callout.show(calloutContainer, _coord, Callout.DIRECTION_UP, false);
					//calloutMessage.arrowOffset = -20;
//					Callout.stagePadding = 20;
					calloutMessage.addEventListener(TouchEvent.TOUCH, function(e:TouchEvent){
						var touchCallout:Touch = e.getTouch(calloutMessage);
						if (touchCallout)
						{
							if (touchCallout.phase == TouchPhase.BEGAN)
							{
								if (checkIfUnitIsInFrame(false)) 
								{
									map.moveMapTo(new Point(x,y));
								}
								else
								{
									calloutMessage.close();
								}
							}
						}
					});
					
					calloutLabel.invalidate();
					//calloutLabel.validate();
					
					if (calloutLabel.width > stage.stageWidth/4)
					{
						calloutLabel.textRendererProperties.wordWrap = true;
						calloutLabel.width = stage.stageWidth/4;
					}
					else 
					{
						calloutLabel.textRendererProperties.wordWrap = false;
	//					calloutLabel.width = stage.stageWidth/5;
					}
		//			calloutMessage. = - width/2;
					//calloutMessage.disposeContent = false;
					if (_useNewOrigin)
					{
						addChild(originSprite);
						originSprite.x = 0;
						originSprite.y = - 30;
						calloutMessage.origin = originSprite;
//						calloutMessage.padding = 5;
						calloutMessage.supportedDirections = Callout.DIRECTION_UP;
					}
					else
					{
						calloutMessage.origin = _coord;
						calloutMessage.supportedDirections = Callout.DIRECTION_UP;
					}
					
					if (!_autoDispose)
					{
						calloutMessage.closeOnTouchBeganOutside = false;
						calloutMessage.closeOnTouchEndedOutside = false;
					}
				}
			}
			//map.moveMapTo(new Point(_coord.x, _coord.y));
		}
		
		private function checkIfUnitIsInFrame(_autoPopup:Boolean = true):Boolean
		{
//			trace("in frame: x:" + localToGlobal(new Point(x,y)));
//			trace("in frame: x:" + ((0 < (this.x * map.scaleX + map.x) && (this.x * map.scaleX + map.x) < stage.stageWidth) && (0 < (this.y * map.scaleY + map.y) && (this.y * map.scaleY + map.y) < stage.stageHeight)));
			previousCheckIfUnitIsInFrame = currentCheckIfUnitIsInFrame;
			currentCheckIfUnitIsInFrame = !((0 < (this.x * map.scaleX + map.x) && (this.x * map.scaleX + map.x) < stage.stageWidth) && (0 < (this.y * map.scaleY + map.y) && (this.y * map.scaleY + map.y) < stage.stageHeight)); 
			
			if (_autoPopup)
			{
				if (currentCheckIfUnitIsInFrame == true && previousCheckIfUnitIsInFrame == false)
				{
					unitIconMini.x = unitIconMini.width/2;
					unitIconMini.y = unitIconMini.height/2;
					//trace("iconUnitContainer.width = " + iconUnitContainer.width);
//					TweenMax.to(calloutContainer, 0, {scale:0.3});
//					iconUnitContainer.scaleX = 0.3;
//					iconUnitContainer.scaleY = 0.3;
//					calloutContainer.removeChildren();
					calloutContainer.addChildAt(unitIconMini,0);
//					calloutContainer.width = 50
					
					if (calloutMessage) calloutMessage.close();
					addChild(originSprite2);
					calloutMessage = Callout.show(calloutContainer,originSprite2, Callout.DIRECTION_DOWN, false);
//					calloutMessage.scaleX = calloutMessage.scaleY = 0.5;
//					calloutMessage.width = 50;
//					calloutMessage.height = 50;
//					calloutMessage.
					//calloutMessage.padding = 0;
					//trace("remove arrow");
					calloutMessage.bottomArrowSkin.removeFromParent();
					calloutMessage.topArrowSkin.removeFromParent();
					calloutMessage.leftArrowSkin.removeFromParent();
					calloutMessage.rightArrowSkin.removeFromParent();
					
					
					calloutMessage.closeOnTouchBeganOutside = false;
					calloutMessage.closeOnTouchEndedOutside = false;
					calloutMessage.addEventListener(TouchEvent.TOUCH, function(e:TouchEvent){
						var touchCallout:Touch = e.getTouch(calloutMessage);
						if (touchCallout)
						{
							if (touchCallout.phase == TouchPhase.BEGAN)
							{
								if (checkIfUnitIsInFrame(false)) 
								{
									map.moveMapTo(new Point(x,y));
								}
								else
								{
									calloutMessage.close();
								}
							}
						}
					});
				}
				else if (currentCheckIfUnitIsInFrame == false && previousCheckIfUnitIsInFrame == true)
				{
//					iconUnitContainer.x = 0;
//					iconUnitContainer.y = 0;
////					iconUnitContainer.scaleX = iconUnitContainer.scaleY = 0.3;
////					iconUnitContainer.scaleX = 1;
////					iconUnitContainer.scaleY = 1;
//					this.addChild(iconUnitContainer);
					calloutMessage.close();
				}
			}
			
			if (currentCheckIfUnitIsInFrame == false && calloutMessage)
			{
				calloutMessage.bottomArrowSkin.removeFromParent();
				calloutMessage.topArrowSkin.removeFromParent();
				calloutMessage.leftArrowSkin.removeFromParent();
				calloutMessage.rightArrowSkin.removeFromParent();
			}
			
			return currentCheckIfUnitIsInFrame;
		}
		
		protected function onTouchTargetPoint(e:TouchEvent):void
		{
			touchUnit = e.getTouch(targetPoint);
			
			
			if (touchUnit)
			{
				if (touchUnit.phase == TouchPhase.BEGAN)
				{
//					trace("Unit.onTouchTargetPoint(): " + (touchUnit.target == targetPoint));
//					trace(" > target: " + touchUnit.target);
//					trace(" > phase: " + touchUnit.phase);
//					trace(" > onMouseDownTargetPoint()");
					onMouseDownTargetPoint();
				}
				else if (touchUnit.phase == TouchPhase.ENDED)
				{
//					trace("Unit.onTouchTargetPoint(): " + (touchUnit.target == targetPoint));
//					trace(" > target: " + touchUnit.target);
//					trace(" > phase: " + touchUnit.phase);
//					trace(" > onMouseUpTargetPoint()");
					onMouseUpTargetPoint();
				}
				else if (touchUnit.phase == TouchPhase.MOVED)
				{
					onMouseMoveTargetPoint();
				}
			}
		}
		
		protected function rotateUnitAlongPath(_isRotateUnitIcon:Boolean = true)
		{
			// Rotate unit smoothly
			if (targetRotation > currentRotation * 180 / Math.PI + 180) targetRotation -= 360;
			if (targetRotation < currentRotation * 180 / Math.PI - 180) targetRotation += 360;
			dr = (targetRotation * Math.PI / 180) - currentRotation;
			ar = dr * rotationSpring;  
			vr += ar;  
			vr *= rotationDamping;
			currentRotation += vr;
			
			if (_isRotateUnitIcon)
			{
				unitIcon.rotation = currentRotation;
				unitIconMini.rotation = currentRotation;
				unitIconShadow.rotation = currentRotation;
			}
			
			if (namePopup)
			{
				//namePopup.validate();
				namePopup.pivotX = 0;//-namePopup.width/2;
				namePopup.pivotY = 0;//-namePopup.height/2;
				namePopup.x = -(35 + namePopup.width/2) * Math.cos(currentRotation);
				namePopup.y = -(35 + namePopup.height/2) * Math.sin(currentRotation);
				
				countdownPopup.pivotX = 0;//-namePopup.width/2;
				countdownPopup.pivotY = 0;//-namePopup.height/2;
				countdownPopup.x = (35 + countdownPopup.width/2) * Math.cos(currentRotation);
				countdownPopup.y = (35 + countdownPopup.height/2) * Math.sin(currentRotation);
				
				
			}
		}
		
		protected function onMouseMoveUnit():void
		{
			if (isDragAndDrop)
			{
				var point:Point = new Point((touchUnit.globalX - map.x)/map.scaleX,(touchUnit.globalY - map.y)/ map.scaleY);
				x = point.x;
				y = point.y;
				targetPoint.x = point.x;
				targetPoint.y = point.y;
			}
		}
		
		protected function onMouseUpUnit():void
		{
//			if (!isDragAndDrop)
//			{
				trace("Unit.onMouseUpUnit()");
				linePath.appendPoint(new Point(targetPoint.x, targetPoint.y));
				updateSpeedAndProgress();
				isCollect = false;
				sendCounter = 30;
				sendUpdatedUnit();
				map.closeMinimap();
//			}
			updateUnitOnNewTerrain();
			
			isDragAndDrop = false;
			
		}
		
		protected function onMouseDownUnit():void
		{
//			trace("/nUnit.onMouseDownUnit() " + isDragAndDrop + " : " + isMyUnit);
			if (!isDragAndDrop)
			{
				map.lineLayer.addChild(targetPoint);
				clearLine();
				progress = 0;
				linePath.points = new Array();
				linePath.appendPoint(new Point(x,y));
				newLinePathPoints = new Array();
				isCollect = true;
				isNewLinepath = true;			
				map.openMinimap();
				
			}
			
			if (calloutMessage) calloutMessage.close();
			
		}
		
		protected function onMouseUpTargetPoint():void
		{
//			if (!isDragAndDrop)
//			{
				trace("Unit.onMouseUpTargetPoint()");
				linePath.appendPoint(new Point(targetPoint.x, targetPoint.y));
				updateSpeedAndProgress();
				isCollect = false;
				sendCounter = 30;
				sendUpdatedUnit();
				map.closeMinimap();
//			}
		}
		
		protected function onMouseMoveTargetPoint():void
		{
			
		}
		
		protected function onMouseDownTargetPoint():void
		{
//			if (!isDragAndDrop)
//			{
				newLinePathPoints = new Array();
				isCollect = true;
				map.openMinimap();
//			}
		}
		
		public function isTerrainNearby(_type:int, _spread:int, _state:int = -1):Boolean
		{
			var _isTerrainNearby:Boolean = false;
			
			//trace("spray water");
			var i, j:Number;
			var _terrain:Object;
			var locX:int = Math.floor(this.x / map.blitSize);
			var locY:int = Math.floor(this.y / map.blitSize);
			
			var spread:Number = _spread;
			
			for (i = -spread; i < spread; i++)
			{
				for (j = -spread; j < spread; j++)
				{
					try
					{
						_terrain = map.terrain[locX + i][locY + j];
						if (_terrain.type == _type)
						{
							_isTerrainNearby = true;
							break;
						}
						if (_terrain.state == _state)
						{
							_isTerrainNearby = true;
							break;
						}
					}
					catch (err:Error)
					{
					}
				}
			}
			
			return _isTerrainNearby;
		}
		
		public function isTerrainArrayNearby(_typeArray:Array, _spread:int, _state:int = -1):Boolean
		{
			var _isTerrainNearby:Boolean = false;
			
			//trace("spray water");
			var i, j, k:Number;
			var _terrain:Object;
			var locX:int = Math.floor(this.x / map.blitSize);
			var locY:int = Math.floor(this.y / map.blitSize);
			
			var spread:Number = _spread;
			
			for (i = -spread; i < spread; i++)
			{
				for (j = -spread; j < spread; j++)
				{
					try
					{
						_terrain = map.terrain[locX + i][locY + j];
						for (k = 0; k < _typeArray.length; k++)
						{
							if (_terrain.type == _typeArray[k])
							{
								_isTerrainNearby = true;
								break;
							}
							if (_state != -1 && _terrain.state == TerrainState.CUT)
							{
								_isTerrainNearby = true;
								break;
							}
						}
					}
					catch (err:Error)
					{
					}
				}
			}
			
			return _isTerrainNearby;
		}
		
		override public function dispose():void
		{
			
			super.dispose();
		}
	}
}