package units
{
	import com.greensock.TweenMax;
	import com.greensock.motionPaths.LinePath2D;
	
	import flash.geom.Point;
	import flash.utils.Dictionary;
	
	import connections.PlayerIOConnection;
	
	import maps.GameDataType;
	import maps.Map;
	import maps.TerrainState;
	import maps.TerrainType;
	
	import org.gestouch.events.GestureEvent;
	import org.gestouch.gestures.TapGesture;
	
	import starling.animation.IAnimatable;
	import starling.core.Starling;
	import starling.display.Image;
	import starling.display.MovieClip;
	import starling.display.Sprite;
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.filters.BlurFilter;
	import starling.filters.ColorMatrixFilter;
	import treefortress.sound.SoundAS;
	
	public class Zombie extends Unit
	{
		private var tappedOnce:Boolean = false;
		private var isGroundUnitNearBy:Boolean = false;
		private var closestGroundUnit:Unit;
		private var closestGroundUnitAngle:Number = 0;
		private var oldGroundUnit:Unit;
		
		
		public function Zombie(_map:Map, _isMyUnit:Boolean, _id:int)
		{
			type = UnitType.ZOMBIE;
			super(_map, _isMyUnit, _id);
		}
		
		override public function toggleNight():void
		{
			super.initializeUnitIcon((map.isNight) ? "ZombieNight" : "Zombie", false);
		}
		
		override public function initializeUnit():void
		{
			super.initializeUnit();
			
			callout("Place me anywhere", this, true, false);
		}

		override protected function initializeOnce():void
		{
			// Add dots to use for line path
			linePointArray = new Array();
			for (var i:int = 0; i < numDots; i++)
			{
				linePointArray.push(new Image(Assets.getAtlas().getTexture("dot")));
				map.lineLayer.addChild(linePointArray[i]);
			}
			
			speed = 0.2;//redblox.randomizeNumber(15,30)/100; // Random speed for variety
			state = UnitState.NORMAL;
			
			
			super.initializeUnitIcon((map.isNight) ? "ZombieNight" : "Zombie", false);
			addNewPointType("targetNormal");
			showPointType("targetNormal");
						
			unitIconShadow.x = 3;
			unitIconShadow.y = 3;
			
			rotateUnitAlongPath(false);
		}
		
		override public function reset():void
		{
			super.reset();
			state = UnitState.NORMAL;
//			if (isMyUnit) tapGesture.removeEventListener(GestureEvent.GESTURE_RECOGNIZED, onDoubleTapUnitIcon);

		}
		
		public function checkIfGroundUnitNearby():void
		{
			var _isGroundUnitNearBy:Boolean = false;
			
			var currentUnit:Unit;
			
			closestGroundUnit = null;
			
			for (var i:int = 0; i < map.unitList.length; i++)
			{
				currentUnit = map.unitList[i];
				if (!currentUnit.isDragAndDrop && (currentUnit.type == UnitType.CIVILIAN || currentUnit.type == UnitType.CUT_TEAM || currentUnit.type == UnitType.HOSE_TEAM || currentUnit.type == UnitType.BULLDOZER || currentUnit.type == UnitType.ENGINE || currentUnit.type == UnitType.BRUSH_TRUCK  || currentUnit.type == UnitType.FIRE_BOAT))
				{
					if (Point.distance(new Point(currentUnit.x, currentUnit.y), new Point(x,y)) < 300)
					{
						if (closestGroundUnit == null)
						{
							closestGroundUnit = currentUnit;
						}
						else if (Point.distance(new Point(currentUnit.x, currentUnit.y), new Point(x,y)) < Point.distance(new Point(closestGroundUnit.x, closestGroundUnit.y), new Point(x,y)))
						{
							closestGroundUnit = currentUnit;
							_isGroundUnitNearBy = true;
							break;
						}
					}
				}
			}
			
			if (oldGroundUnit != closestGroundUnit)
			{
				oldGroundUnit = closestGroundUnit;
//				if (closestGroundUnit) 
					SoundAS.play("Zombie");
			}
			
			if (closestGroundUnit && Point.distance(new Point(closestGroundUnit.x, closestGroundUnit.y), new Point(x,y)) < 15)
			{
				callout("Gotchya!", this, true);
				SoundAS.play("Scream", 0.5);
//				var zombie:Unit = map.addUnit(UnitType.ZOMBIE, map.currentZombieID, true);
				if (map.playerioConnection && map.isGameMaster()) map.playerioConnection.send("addUnit", UnitType.ZOMBIE, -1, Math.round(closestGroundUnit.x) + redblox.randomizeNumber(-25, 25), Math.round(closestGroundUnit.y) + redblox.randomizeNumber(-25, 25) );
				
//				zombie.isDragAndDrop = false;
//				zombie.x = closestGroundUnit.x + redblox.randomizeNumber(-50, 50);
//				zombie.y = closestGroundUnit.y + redblox.randomizeNumber(-50, 50);
//				if (zombie.calloutMessage) zombie.calloutMessage.close();
				closestGroundUnit.reset();
			}
			
//			if (_isGroundUnitNearBy && !isGroundUnitNearBy)
//			{
//				unitIcon.currentFrame = 1;
//				isGroundUnitNearBy = true;
//				var answers:Array = new Array();
//				answers.push("There's a fire nearby?! Where should I go?");
//				answers.push("I'm lost, can you show me the way out?");
//				answers.push("Do I have to go? Fine, where to?");
//				answers.push("Help me! Save me!");
//				answers.push("Which way should I go?");
//				callout(answers[redblox.randomizeNumber(0,answers.length-1)], this, true);
//			}
////			else if (!_isGroundUnitNearBy && isGroundUnitNearBy)
//			{
//				unitIcon.currentFrame = 0;
//				isGroundUnitNearBy = false;
//
//			}
		}
		
		override public function enable():void
		{
			Starling.juggler.add(this);
//			unitIcon.currentFrame = 0;
//			isGroundUnitNearBy = false;
			//map.unitLayer.addChild(this);
			
		}
		
		override public function advanceTime(passedTime:Number):void
		{
			//rotateUnitAlongPath(false);
			originSprite.x = x;
			originSprite.y = y - 20;
			
			if (isDragAndDrop)
			{
				return;
			}
			
			checkIfGroundUnitNearby();		
			
			if (closestGroundUnit == null || state == UnitState.DEAD)
			{
				return;
			}
			
			updateUnitOnNewTerrain();
				
			if (this.x > 0 && this.x < map.terrainWidth && this.y > 0 && this.y < map.terrainHeight)
			{
				closestGroundUnitAngle = redblox.point_direction(this.x, this.y, closestGroundUnit.x, closestGroundUnit.y) * Math.PI / 180;
				
				this.x += speed * (passedTime * 30) * Math.cos(closestGroundUnitAngle) * TweenMax.globalTimeScale * speedVariantBasedOnTerrain;
				this.y += speed * (passedTime * 30) * Math.sin(closestGroundUnitAngle) * TweenMax.globalTimeScale * speedVariantBasedOnTerrain;
			}
		}
		
		override protected function updateUnitOnNewTerrain():void
		{
			super.updateUnitOnNewTerrain();
			
			//speed = UnitSpeeds[state][terrainUnderUnit.type][terrainUnderUnit.state]
			
			if (terrainUnderUnit && terrainUnderUnit != terrainUnderUnitPrevious)
			{
				if (state == UnitState.NORMAL)
				{
					if (terrainUnderUnit == null)
					{
						speedVariantBasedOnTerrain = 10;
					}
					else if (terrainUnderUnit.state == TerrainState.FIRE || terrainUnderUnit.state == TerrainState.FIRE_WET)
					{
						//map.gameOverPanel("Civilian died!");
						speedVariantBasedOnTerrain = 0;
						SoundAS.play("Zombie");
						TweenMax.to(this.unitIcon, 1, {rotation:80 * Math.PI/180,onComplete:reset});
					}
					else if (terrainUnderUnit.state == TerrainState.DEAD || terrainUnderUnit.state == TerrainState.DEAD_WET)
						speedVariantBasedOnTerrain = 0.5;
					else if (terrainUnderUnit.state == TerrainState.BURNT || terrainUnderUnit.state == TerrainState.BURNT_WET)
						speedVariantBasedOnTerrain = 0.8;
					else if (terrainUnderUnit.state == TerrainState.CUT || terrainUnderUnit.state == TerrainState.CUT_WET)
						speedVariantBasedOnTerrain = 1;
					else if (terrainUnderUnit.type == TerrainType.WATER_DEEP)
					{
						SoundAS.play("Zombie");
						TweenMax.to(this.unitIcon, 1, {rotation:80 * Math.PI/180,onComplete:reset});
						speedVariantBasedOnTerrain = 0.05;
					}
					else if (terrainUnderUnit.type == TerrainType.WATER_SHALLOW)
					{
						SoundAS.play("Zombie");
						TweenMax.to(this.unitIcon, 1, {rotation:80 * Math.PI/180,onComplete:reset});
						speedVariantBasedOnTerrain = 0.05;
					}
					else if (terrainUnderUnit.type == TerrainType.SAND)
						speedVariantBasedOnTerrain = 1.25;
					else if (terrainUnderUnit.type == TerrainType.GRAVEL)
						speedVariantBasedOnTerrain = 1.5;
					else if (terrainUnderUnit.type == TerrainType.ROAD)
						speedVariantBasedOnTerrain = 1.5;
					else if (terrainUnderUnit.type == TerrainType.GRASS)
						speedVariantBasedOnTerrain = 1;
					else if (terrainUnderUnit.type == TerrainType.BUSH)
						speedVariantBasedOnTerrain = 0.75; //0.80;
					else if (terrainUnderUnit.type == TerrainType.TREE)
						speedVariantBasedOnTerrain = 0.5; // 0.5;
					else if (terrainUnderUnit.type == TerrainType.MARSH)
						speedVariantBasedOnTerrain = 0.3;
					else
						speedVariantBasedOnTerrain = 1;
				
//					if ((terrainUnderUnitPrevious == undefined || terrainUnderUnitPrevious.type != TerrainType.TREE) && (terrainUnderUnit && terrainUnderUnit.type == TerrainType.TREE))
//					{
//						var answers:Array = new Array();
//						answers.push("These dense trees are slow to cut through!");
//						answers.push("Is there an easier path?");
//						answers.push("A bulldozer would be nice right about now!");
//						answers.push("I'm a lot faster cutting grasslands or bushes!");
//						answers.push("I hope I don't have to cut much more of this dense forest.");
//						callout(answers[redblox.randomizeNumber(0,answers.length-1)], this, true);
//					}
					
					
				}
			}
		}
		
		/**
		 * 
		 * @param	unit
		 */
		override protected function arrivedAtDestination ():void
		{
			//trace("\nUnit.arrivedAtDestination();");
			progress = 0;
			linePath.points = new Array();
			map.lineLayer.removeChild(targetPoint);
			callout("Zombie arrived \nat destination!", this, true);
			unitIcon.currentFrame = 0;
			isGroundUnitNearBy = false;
		} 
		
		
		/**
		 * 
		 * @param	_terrain
		 */
		override public function sendUpdatedUnit(_forceSend:Boolean = false):void 
		{
			if (isMyUnit && (sendCounter >= 5 || _forceSend))
			{
				sendCounter = 0;
				var gameData:Object = new Object();
				gameData.action = GameDataType.UNIT_NEWLINEPATHPOINTS;
				gameData.userID = PlayerIOConnection.myUserID;
				
				gameData.points = linePath.points;
				gameData.speed = speed;
				gameData.speedOnPath = speedOnPath;
				gameData.progress = progress;
				gameData.state = state;
				gameData.speedVariantBasedOnTerrain = speedVariantBasedOnTerrain;
				//				data.isOffScreen = isOffScreen;
				//				data.isEnteringMap = isEnteringMap;
				gameData.id = id;
				gameData.type = type;
				gameData.oldLength = oldLength;
				gameData.oldProgress = oldProgress;
				gameData.x = x;
				gameData.y = y;
				gameData.rotation = unitIcon.rotation;
				gameData.targetPointRotation = targetPoint.rotation;
				gameData.pointName = pointName;
				gameData.playerName = PlayerIOConnection.playerName;
				
				map.sendDataToPlayerIO(gameData);
			}
			sendCounter++;
		}
		
		override public function processUnit(_data:Object):void
		{
			//			if (map.isSetupComplete)
			if (true)
			{
//				checkIfGroundUnitNearby();
				
				var tempPoints:Array = new Array();
				for (var i:int = 0; i < _data.points.length; i++)
				{
					tempPoints.push(new Point(_data.points[i].x, _data.points[i].y))
				}
				linePath.points = tempPoints;
				speed = _data.speed;
				speedOnPath = _data.speedOnPath;
				progress = _data.progress;
				oldLength = _data.oldLength;
				oldProgress = _data.oldProgress;
				unitIcon.rotation = _data.rotation;
				targetPoint.rotation = _data.targetPointRotation;
				speedVariantBasedOnTerrain = _data.speedVariantBasedOnTerrain;
				state = _data.state;
				
//				if (state == UnitState.NORMAL) 
//				{
//					unitIcon.currentFrame = 0;
//				}
//				else
//				{
//					unitIcon.currentFrame = 1;
//				}

				if (linePath.points.length > 0)
				{
					targetPoint.x = linePath.points[linePath.points.length-1].x;
					targetPoint.y = linePath.points[linePath.points.length-1].y;
					map.lineLayer.addChild(targetPoint);
				}
				else
				{
					if (map.lineLayer.contains(targetPoint)) map.lineLayer.removeChild(targetPoint);
				}
				//this.img.icon.visible = _data.visible;
				udpatePlayerName(_data.playerName);
				updateSpeedAndProgress();
				//map.lineLayer.addChild(targetPoint);
				showPointType(_data.pointName);
				updatePointTypeByTerrain();
				clearLine();
				this.x = _data.x;
				this.y = _data.y;
				drawLineCounter = 10;
				drawLine();
				isDragAndDrop = false;
				this.visible = true;
				if (map.unitLayer.contains(this) == false) map.unitLayer.addChildAt(this,0);

				//if (tempPoints.length > 0) TweenMax.to(endPoint, 0, { x:tempPoints[tempPoints.length - 1].x, y: tempPoints[tempPoints.length - 1].y, autoAlpha:1 } );
				
				if (targetUnit) // check to see if a helicopter is coming to pick him up
				{
					targetUnit.disable(false);
					targetUnit.targetUnit = null;
					targetUnit = null;
				}
			}
		}
		override protected function onMouseDownTargetPoint():void
		{
			if (isGroundUnitNearBy)
			{
				newLinePathPoints = new Array();
				isCollect = true;
				map.openMinimap();
			}
		}
		
		override protected function onMouseUpUnit():void
		{
			if (isDragAndDrop) map.unitLayer.addChildAt(this,0);
			super.onMouseUpUnit();
		}
			
		
		override protected function onMouseDownUnit():void
		{
//			super.onMouseDownUnit();
			if (!isDragAndDrop && isGroundUnitNearBy)
			{
				map.lineLayer.addChild(targetPoint);
				clearLine();
				progress = 0;
				linePath.points = new Array();
				linePath.appendPoint(new Point(x,y));
				newLinePathPoints = new Array();
				isCollect = true;
				
				map.openMinimap();
			}
			speedVariantBasedOnTerrain = 1;
			
//			if (!isGroundUnitNearBy)
//			{
//				callout("Can't move without guidance!", );
//			}

			if (targetUnit) // check to see if a helicopter is coming to pick him up
			{
				targetUnit.disable(false);
				targetUnit.targetUnit = null;
				targetUnit = null;
			}
		}
		
		override protected function onMouseMoveUnit():void
		{
			tappedOnce = false;
			super.onMouseMoveUnit();
		}		
		
		
		override public function dispose():void
		{
			
			super.dispose();
		}
	}
}